@@ -83,7 +83,11 @@ Proxy::Proxy(SipStack& stack,
      mRecordRouteForced(config.getConfigBool("ForceRecordRouting", false)),
      mAssumePath(config.getConfigBool("AssumePath", false)),
      mPAssertedIdentityProcessing(config.getConfigBool("EnablePAssertedIdentityProcessing", false)),
+#ifdef PACKAGE_VERSION
+     mServerText(config.getConfigData("ServerText", "repro " PACKAGE_VERSION)),
+#else
      mServerText(config.getConfigData("ServerText", "")),
+#endif
      mTimerC(config.getConfigInt("TimerC", 180)),
      mKeyValueStore(*Proxy::getGlobalKeyValueStoreKeyAllocator()),
      mRequestProcessorChain(requestP), 
@@ -91,7 +95,10 @@ Proxy::Proxy(SipStack& stack,
      mTargetProcessorChain(targetP),
      mUserStore(config.getDataStore()->mUserStore),
      mOptionsHandler(0),
-     mRequestContextFactory(new RequestContextFactory)
+     mRequestContextFactory(new RequestContextFactory),
+     mSessionAccountingEnabled(config.getConfigBool("SessionAccountingEnabled", false)),
+     mRegistrationAccountingEnabled(config.getConfigBool("RegistrationAccountingEnabled", false)),
+     mAccountingCollector(0)
 {
    FlowTokenSalt = Random::getCryptoRandom(20);   // 20-octet Crypto Random Key for Salting Flow Token HMACs
 
@@ -101,6 +108,12 @@ Proxy::Proxy(SipStack& stack,
    {
       addSupportedOption("outbound");
    }
+
+   // Create Accounting Collector if enabled
+   if(mSessionAccountingEnabled || mRegistrationAccountingEnabled)
+   {
+      mAccountingCollector = new AccountingCollector(config);
+   }
 }
 
 
@@ -108,6 +121,7 @@ Proxy::~Proxy()
 {
    shutdown();
    join();
+   delete mAccountingCollector;
    InfoLog (<< "Proxy::thread shutdown with " << mServerRequestContexts.size() << " ServerRequestContexts and " << mClientRequestContexts.size() << " ClientRequestContexts.");
 }
 
@@ -621,6 +635,25 @@ Proxy::removeSupportedOption(const resip::Data& option)
    mSupportedOptions.erase(option);
 }
 
+void 
+Proxy::doSessionAccounting(const resip::SipMessage& sip, bool received, RequestContext& context)
+{
+   if(mSessionAccountingEnabled)
+   {
+      assert(mAccountingCollector);
+      mAccountingCollector->doSessionAccounting(sip, received, context);
+   }
+}
+
+void 
+Proxy::doRegistrationAccounting(AccountingCollector::RegistrationEvent regEvent, const resip::SipMessage& sip)
+{
+   if(mRegistrationAccountingEnabled)
+   {
+      assert(mAccountingCollector);
+      mAccountingCollector->doRegistrationAccounting(regEvent, sip);
+   }
+}
 
 
 /* ====================================================================
