@@ -5,24 +5,44 @@
 #include <iostream>
 #include <fstream>
 #include <stdexcept>
+#ifndef WIN32
+#include <syslog.h>
+#endif
+
+#ifdef REPRO_DSO_PLUGINS
+
+// in an autotools build, this is defined using pkglibdir
+#ifndef REPRO_DSO_PLUGIN_DIR_DEFAULT
+#define REPRO_DSO_PLUGIN_DIR_DEFAULT ""
+#endif
+
+// This is the UNIX way of doing DSO, an alternative implementation
+// for Windows needs to include the relevant Windows headers here
+// and implement the loader code further below
+#include <dlfcn.h>
+
+#endif
 
 #include "rutil/Log.hxx"
 #include "rutil/Logger.hxx"
 #include "rutil/DnsUtil.hxx"
 #include "rutil/dns/DnsStub.hxx"
 #include "rutil/GeneralCongestionManager.hxx"
+#include "rutil/TransportType.hxx"
 
 #include "resip/stack/SipStack.hxx"
 #include "resip/stack/Compression.hxx"
 #include "resip/stack/EventStackThread.hxx"
 #include "resip/stack/InteropHelper.hxx"
 #include "resip/stack/ConnectionManager.hxx"
+#include "resip/stack/WsCookieContextFactory.hxx"
 
 #include "resip/dum/InMemorySyncRegDb.hxx"
 #include "resip/dum/MasterProfile.hxx"
 #include "resip/dum/DialogUsageManager.hxx"
 #include "resip/dum/DumThread.hxx"
 #include "resip/dum/TlsPeerAuthManager.hxx"
+#include "resip/dum/WsCookieAuthManager.hxx"
 
 #include "repro/AsyncProcessorWorker.hxx"
 #include "repro/ReproRunner.hxx"
@@ -36,12 +56,15 @@
 #include "repro/WebAdmin.hxx"
 #include "repro/WebAdminThread.hxx"
 #include "repro/Registrar.hxx"
+#include "repro/ReproAuthenticatorFactory.hxx"
 #include "repro/ReproServerAuthManager.hxx"
 #include "repro/RegSyncClient.hxx"
 #include "repro/RegSyncServer.hxx"
 #include "repro/RegSyncServerThread.hxx"
 #include "repro/CommandServer.hxx"
 #include "repro/CommandServerThread.hxx"
+#include "repro/BasicWsConnectionValidator.hxx"
+#include "repro/monkeys/CookieAuthenticator.hxx"
 #include "repro/monkeys/IsTrustedNode.hxx"
 #include "repro/monkeys/AmIResponsible.hxx"
 #include "repro/monkeys/DigestAuthenticator.hxx"
@@ -102,7 +125,6 @@ ReproRunner::ReproRunner()
    : mRunning(false)
    , mRestarting(false)
    , mThreadedStack(false)
-   , mSipAuthDisabled(false)
    , mUseV4(true)
    , mUseV6 (false)
    , mRegSyncPort(0)
@@ -114,13 +136,12 @@ ReproRunner::ReproRunner()
    , mAbstractDb(0)
    , mRuntimeAbstractDb(0)
    , mRegistrationPersistenceManager(0)
-   , mAuthRequestDispatcher(0)
+   , mAuthFactory(0)
    , mAsyncProcessorDispatcher(0)
    , mMonkeys(0)
    , mLemurs(0)
    , mBaboons(0)
    , mProxy(0)
-   , mWebAdminList(0)
    , mWebAdminThread(0)
    , mRegistrar(0)
    , mDum(0)
@@ -130,7 +151,6 @@ ReproRunner::ReproRunner()
    , mRegSyncServerV4(0)
    , mRegSyncServerV6(0)
    , mRegSyncServerThread(0)
-   , mCommandServerList(0)
    , mCommandServerThread(0)
    , mCongestionManager(0)
 {
@@ -164,6 +184,9 @@ ReproRunner::run(int argc, char** argv)
    catch(BaseException& ex)
    {
       std::cerr << "Error parsing configuration: " << ex << std::endl;
+#ifndef WIN32
+      syslog(LOG_DAEMON | LOG_CRIT, "%s", ex.getMessage().c_str());
+#endif
       return false;
    }
 
@@ -180,10 +203,9 @@ ReproRunner::run(int argc, char** argv)
    // Initialize resip logger
    GenericLogImpl::MaxByteCount = mProxyConfig->getConfigUnsignedLong("LogFileMaxBytes", 5242880 /*5 Mb */);
    Data loggingType = mProxyConfig->getConfigData("LoggingType", "cout", true);
-   Data mAppName = mProxyConfig->getConfigData("LoggingInstanceName", mArgv[0], true);
    Log::initialize(loggingType, 
                    mProxyConfig->getConfigData("LogLevel", "INFO", true), 
-                   mAppName, 
+                   mArgv[0], 
                    mProxyConfig->getConfigData("LogFilename", "repro.log", true).c_str(),
                    isEqualNoCase(loggingType, "file") ? &g_ReproLogger : 0); // if logging to file then write WARNINGS, and Errors to console still
 
@@ -195,6 +217,12 @@ ReproRunner::run(int argc, char** argv)
       return false;
    }
 
+   // Load the plugins after creating the stack, as they may need it
+   if(!loadPlugins())
+   {
+      return false;
+   }
+
    // Drop privileges (can do this now that sockets are bound)
    Data runAsUser = mProxyConfig->getConfigData("RunAsUser", "", true);
    Data runAsGroup = mProxyConfig->getConfigData("RunAsGroup", "", true); 
@@ -210,6 +238,9 @@ ReproRunner::run(int argc, char** argv)
       return false;
    }
 
+   // Create authentication mechanism
+   createAuthenticatorFactory();
+
    // Create DialogUsageManager that handles ServerRegistration,
    // and potentially certificate subscription server
    createDialogUsageManager();
@@ -314,12 +345,13 @@ ReproRunner::shutdown()
    {
       mDumThread->join();
    }
-   if(mAuthRequestDispatcher)
+   if(mAuthFactory)
    {
       // Both proxy and dum threads are down at this point, we can 
-      // destroy the authRequest dispatcher and associated threads now
-      delete mAuthRequestDispatcher;
-      mAuthRequestDispatcher = 0;
+      // destroy the authFactory and its authRequest dispatcher
+      // and associated threads now
+      delete mAuthFactory;
+      mAuthFactory = 0;
    }
    if(mAsyncProcessorDispatcher)
    {
@@ -357,6 +389,17 @@ ReproRunner::restart()
    mRestarting = false;
 }
 
+void
+ReproRunner::onHUP()
+{
+   // Let the plugins know
+   std::vector<Plugin*>::iterator it;
+   for(it = mPlugins.begin(); it != mPlugins.end(); it++)
+   {
+      (*it)->onReload();
+   }
+}
+
 void
 ReproRunner::cleanupObjects()
 {
@@ -365,11 +408,11 @@ ReproRunner::cleanupObjects()
    {
       // We leave command server running during restart
       delete mCommandServerThread; mCommandServerThread = 0;
-      for(std::list<CommandServer*>::iterator it = mCommandServerList->begin(); it != mCommandServerList->end(); it++)
+      for(std::list<CommandServer*>::iterator it = mCommandServerList.begin(); it != mCommandServerList.end(); it++)
       {
          delete (*it);
       }
-      delete mCommandServerList; mCommandServerList = 0;
+      mCommandServerList.clear();
    }
    delete mRegSyncServerThread; mRegSyncServerThread = 0;
    delete mRegSyncServerV6; mRegSyncServerV6 = 0;
@@ -382,16 +425,16 @@ ReproRunner::cleanupObjects()
    delete mDum; mDum = 0;
    delete mRegistrar; mRegistrar = 0;
    delete mWebAdminThread; mWebAdminThread = 0;
-   for(std::list<WebAdmin*>::iterator it = mWebAdminList->begin(); it != mWebAdminList->end(); it++)
+   for(std::list<WebAdmin*>::iterator it = mWebAdminList.begin(); it != mWebAdminList.end(); it++)
    {
       delete (*it);
    }
-   delete mWebAdminList; mWebAdminList = 0;
+   mWebAdminList.clear();
    delete mProxy; mProxy = 0;
    delete mBaboons; mBaboons = 0;
    delete mLemurs; mLemurs = 0;
    delete mMonkeys; mMonkeys = 0;
-   delete mAuthRequestDispatcher; mAuthRequestDispatcher = 0;
+   delete mAuthFactory; mAuthFactory = 0;
    delete mAsyncProcessorDispatcher; mAsyncProcessorDispatcher = 0;
    if(!mRestarting) 
    {
@@ -407,6 +450,77 @@ ReproRunner::cleanupObjects()
    delete mProxyConfig; mProxyConfig = 0;
 }
 
+bool
+ReproRunner::loadPlugins()
+{
+   std::vector<Data> pluginNames;
+   mProxyConfig->getConfigValue("LoadPlugins", pluginNames);
+
+#ifdef REPRO_DSO_PLUGINS
+   if(pluginNames.empty())
+   {
+      DebugLog(<<"LoadPlugins not specified, not attempting to load any plugins");
+      return true;
+   }
+
+   const Data& pluginDirectory = mProxyConfig->getConfigData("PluginDirectory", REPRO_DSO_PLUGIN_DIR_DEFAULT, true);
+   if(pluginDirectory.empty())
+   {
+      ErrLog(<<"LoadPlugins specified but PluginDirectory not specified, can't load plugins");
+      return false;
+   }
+   for(std::vector<Data>::iterator it = pluginNames.begin(); it != pluginNames.end(); it++)
+   {
+      void *dlib;
+      // FIXME:
+      // - not all platforms use the .so extension
+      // - detect and use correct directory separator charactor
+      // - do we need to support relative paths here?
+      // - should we use the filename prefix 'lib', 'mod' or something else?
+      Data name = pluginDirectory + '/' + "lib" + *it + ".so";
+      dlib = dlopen(name.c_str(), RTLD_NOW | RTLD_GLOBAL);
+      if(!dlib)
+      {
+         ErrLog(<< "Failed to load plugin " << *it << " (" << name << "): " << dlerror());
+         return false;
+      }
+      ReproPluginDescriptor* desc = (ReproPluginDescriptor*)dlsym(dlib, "reproPluginDesc");
+      if(!desc)
+      {
+         ErrLog(<< "Failed to find reproPluginDesc in plugin " << *it << " (" << name << "): " << dlerror());
+         return false;
+      }
+      if(!(desc->mPluginApiVersion == REPRO_DSO_PLUGIN_API_VERSION))
+      {
+         ErrLog(<< "Failed to load plugin " << *it << " (" << name << "): found version " << desc->mPluginApiVersion << ", expecting version " << REPRO_DSO_PLUGIN_API_VERSION);
+      }
+      DebugLog(<<"Trying to instantiate plugin " << *it);
+      // Instantiate the plugin object and add it to our runtime environment
+      Plugin* plugin = desc->creationFunc();
+      if(!plugin)
+      {
+         ErrLog(<< "Failed to instantiate plugin " << *it << " (" << name << ")");
+         return false;
+      }
+      if(!plugin->init(*mSipStack, mProxyConfig))
+      {
+         ErrLog(<< "Failed to initialize plugin " << *it << " (" << name << ")");
+         return false;
+      }
+      mPlugins.push_back(plugin);
+   }
+   return true;
+#else
+   if(!pluginNames.empty())
+   {
+      ErrLog(<<"LoadPlugins specified but repro not compiled with plugin DSO support");
+      return false;
+   }
+   DebugLog(<<"Not compiled with plugin DSO support");
+   return true;
+#endif
+}
+
 bool
 ReproRunner::createSipStack()
 {
@@ -419,6 +533,13 @@ ReproRunner::createSipStack()
       resip::Timer::resetT1(overrideT1);
    }
 
+   unsigned long messageSizeLimit = mProxyConfig->getConfigUnsignedLong("StreamMessageSizeLimit", 0);
+   if(messageSizeLimit > 0)
+   {
+      DebugLog(<< "Using maximum message size "<< messageSizeLimit << " on stream-based transports");
+      ConnectionBase::setMessageSizeMax(messageSizeLimit);
+   }
+
    // Create Security (TLS / Certificates) and Compression (SigComp) objects if
    // pre-precessor defines are enabled
    Security* security = 0;
@@ -513,6 +634,9 @@ ReproRunner::createSipStack()
       mSipStack->setEnumDomains(enumDomains);
    }
 
+   // Add External Stats handler
+   mSipStack->setExternalStatsHandler(this);
+
    // Add stack transports
    bool allTransportsSpecifyRecordRoute=false;
    if(!addTransports(allTransportsSpecifyRecordRoute))
@@ -535,11 +659,22 @@ ReproRunner::createSipStack()
    {
       InteropHelper::setClientNATDetectionMode(InteropHelper::ClientNATDetectionPrivateToPublicOnly);
    }
+   ConnectionManager::MinimumGcHeadroom = mProxyConfig->getConfigUnsignedLong("TCPMinimumGCHeadroom", 0);
+   unsigned long tcpConnectionGCAge = mProxyConfig->getConfigUnsignedLong("TCPConnectionGCAge", 0);
+   if(tcpConnectionGCAge > 0)
+   {
+      ConnectionManager::MinimumGcAge = tcpConnectionGCAge * 1000;
+      ConnectionManager::EnableAgressiveGc = true;
+   }
    unsigned long outboundFlowTimer = mProxyConfig->getConfigUnsignedLong("FlowTimer", 0);
    if(outboundFlowTimer > 0)
    {
       InteropHelper::setFlowTimerSeconds(outboundFlowTimer);
-      ConnectionManager::MinimumGcAge = 7200000; // Timeout connections not related to a flow timer after 2 hours - TODO make configurable
+      if(tcpConnectionGCAge == 0)
+      {
+         // This should be set too when using outboundFlowTimer
+         ConnectionManager::MinimumGcAge = 7200000;
+      }
       ConnectionManager::EnableAgressiveGc = true;
    }
 
@@ -678,6 +813,14 @@ ReproRunner::createDatastore()
    return true;
 }
 
+void
+ReproRunner::createAuthenticatorFactory()
+{
+   // TODO: let a plugin supply an instance of AuthenticatorFactory
+   // instead of our builtin ReproAuthenticatorFactory
+   mAuthFactory = new ReproAuthenticatorFactory(*mProxyConfig, *mSipStack, mDum);
+}
+
 void
 ReproRunner::createDialogUsageManager()
 {
@@ -698,6 +841,15 @@ ReproRunner::createDialogUsageManager()
    {
        profile->allowBadRegistrationEnabled() = true;
    }
+#ifdef PACKAGE_VERSION
+   Data serverText(mProxyConfig->getConfigData("ServerText", "repro " PACKAGE_VERSION));
+#else
+   Data serverText(mProxyConfig->getConfigData("ServerText", ""));
+#endif
+   if(!serverText.empty())
+   {
+      profile->setUserAgent(serverText);
+   }
    
    // Create DialogeUsageManager if Registrar or Certificate Server are enabled
    assert(!mRegistrar);
@@ -752,43 +904,30 @@ ReproRunner::createDialogUsageManager()
 #endif
    }
 
-   mSipAuthDisabled = mProxyConfig->getConfigBool("DisableAuth", false);
-
    if (mDum)
    {
-      bool enableCertAuth = mProxyConfig->getConfigBool("EnableCertificateAuthenticator", false);
-      // Maintains existing behavior for non-TLS cert auth users
-      bool digestChallengeThirdParties = !enableCertAuth;
+      assert(mAuthFactory);
+      mAuthFactory->setDum(mDum);
 
-      if(enableCertAuth)
+      if(mAuthFactory->certificateAuthEnabled())
       {
          // TODO: perhaps this should be initialised from the trusted node
          // monkey?  Or should the list of trusted TLS peers be independent
          // from the trusted node list?
-         std::set<Data> trustedPeers;
-         loadCommonNameMappings();
-         SharedPtr<TlsPeerAuthManager> certAuth(new TlsPeerAuthManager(*mDum, mDum->dumIncomingTarget(), trustedPeers, true, mCommonNameMappings));
-         mDum->addIncomingFeature(certAuth);
+         mDum->addIncomingFeature(mAuthFactory->getCertificateAuthManager());
+      }
+
+      Data wsCookieAuthSharedSecret = mProxyConfig->getConfigData("WSCookieAuthSharedSecret", "");
+      if(!mAuthFactory->digestAuthEnabled() && !wsCookieAuthSharedSecret.empty())
+      {
+         SharedPtr<WsCookieAuthManager> cookieAuth(new WsCookieAuthManager(*mDum, mDum->dumIncomingTarget()));
+         mDum->addIncomingFeature(cookieAuth);
       }
 
       // If Authentication is enabled, then configure DUM to authenticate requests
-      if (!mSipAuthDisabled)
+      if (mAuthFactory->digestAuthEnabled())
       {
-         // Create UserAuthGrabber Worker Thread Pool if auth is enabled
-         assert(!mAuthRequestDispatcher);
-         int numAuthGrabberWorkerThreads = mProxyConfig->getConfigInt("NumAuthGrabberWorkerThreads", 2);
-         if(numAuthGrabberWorkerThreads < 1) numAuthGrabberWorkerThreads = 1; // must have at least one thread
-         std::auto_ptr<Worker> grabber(new UserAuthGrabber(mProxyConfig->getDataStore()->mUserStore));
-         mAuthRequestDispatcher = new Dispatcher(grabber, mSipStack, numAuthGrabberWorkerThreads);
-
-         SharedPtr<ServerAuthManager> 
-            uasAuth( new ReproServerAuthManager(*mDum,
-                                                mAuthRequestDispatcher,
-                                                mProxyConfig->getDataStore()->mAclStore,
-                                                !mProxyConfig->getConfigBool("DisableAuthInt", false) /*useAuthInt*/,
-                                                mProxyConfig->getConfigBool("RejectBadNonces", false),
-                                                digestChallengeThirdParties));
-         mDum->setServerAuthManager(uasAuth);
+         mDum->setServerAuthManager(mAuthFactory->getServerAuthManager());
       }
 
       // Set the MessageFilterRuleList on DUM and create a thread to run DUM in
@@ -811,6 +950,8 @@ ReproRunner::createProxy()
                                                  numAsyncProcessorWorkerThreads);
    }
 
+   std::vector<Plugin*>::iterator it;
+
    // Create proxy processor chains
    /* Explanation:  "Monkeys" are processors which operate on incoming requests
                     "Lemurs"  are processors which operate on incoming responses
@@ -821,18 +962,30 @@ ReproRunner::createProxy()
    mMonkeys = new ProcessorChain(Processor::REQUEST_CHAIN);
    makeRequestProcessorChain(*mMonkeys);
    InfoLog(<< *mMonkeys);
+   for(it = mPlugins.begin(); it != mPlugins.end(); it++)
+   {
+      (*it)->onRequestProcessorChainPopulated(*mMonkeys);
+   }
 
    // Make Lemurs
    assert(!mLemurs);
    mLemurs = new ProcessorChain(Processor::RESPONSE_CHAIN);
    makeResponseProcessorChain(*mLemurs);
    InfoLog(<< *mLemurs);
+   for(it = mPlugins.begin(); it != mPlugins.end(); it++)
+   {
+      (*it)->onResponseProcessorChainPopulated(*mLemurs);
+   }
 
    // Make Baboons
    assert(!mBaboons);
    mBaboons = new ProcessorChain(Processor::TARGET_CHAIN);
    makeTargetProcessorChain(*mBaboons);
    InfoLog(<< *mBaboons);
+   for(it = mPlugins.begin(); it != mPlugins.end(); it++)
+   {
+      (*it)->onTargetProcessorChainPopulated(*mBaboons);
+   }
 
    // Create main Proxy class
    assert(!mProxy);
@@ -841,13 +994,29 @@ ReproRunner::createProxy()
                       *mMonkeys, 
                       *mLemurs, 
                       *mBaboons);
-   mHttpRealm = addDomains(*mProxy, true);
+   Data defaultRealm = addDomains(*mProxy, true);
+   mHttpRealm = mProxyConfig->getConfigData("HttpAdminRealm", defaultRealm);
+
+   // Set Server Text
+#ifdef PACKAGE_VERSION
+   Data serverText(mProxyConfig->getConfigData("ServerText", "repro " PACKAGE_VERSION));
+#else
+   Data serverText(mProxyConfig->getConfigData("ServerText", ""));
+#endif
+   if(!serverText.empty())
+   {
+      mProxy->setServerText(serverText);
+   }
 
    // Register the Proxy class a stack transaction user
    // Note:  This is done after creating the DialogUsageManager so that it acts 
    // like a catchall and will handle all requests the DUM does not
    mSipStack->registerTransactionUser(*mProxy);
 
+   if(mRegistrar)
+   {
+      mRegistrar->setProxy(mProxy);
+   }
    return true;
 }
 
@@ -886,14 +1055,13 @@ ReproRunner::populateRegistrations()
 bool
 ReproRunner::createWebAdmin()
 {
-   assert(!mWebAdminList);
+   assert(mWebAdminList.empty());
    assert(!mWebAdminThread);
 
    std::vector<resip::Data> httpServerBindAddresses;
    mProxyConfig->getConfigValue("HttpBindAddress", httpServerBindAddresses);
    int httpPort = mProxyConfig->getConfigInt("HttpPort", 5080);
 
-   mWebAdminList = new std::list<WebAdmin*>;
    if(httpPort)
    {
       if(httpServerBindAddresses.empty())
@@ -912,48 +1080,70 @@ ReproRunner::createWebAdmin()
       {
          if(mUseV4 && DnsUtil::isIpV4Address(*it)) 
          {
-            WebAdmin* WebAdminV4 = new WebAdmin(*mProxy,
-                                                *mRegistrationPersistenceManager, 
-                                                mHttpRealm, 
-                                                httpPort,
-                                                V4,
-                                                *it);
+            WebAdmin* webAdminV4 = 0;
 
-            if (!WebAdminV4->isSane())
+            try 
+            {
+               webAdminV4 = new WebAdmin(*mProxy,
+                                         *mRegistrationPersistenceManager, 
+                                         mHttpRealm, 
+                                         httpPort,
+                                         V4,
+                                         *it);
+            } 
+            catch(WebAdmin::ConfigException& ex) 
+            {
+               ErrLog(<<"Exception when starting WebAdmin: " << ex.getMessage());
+               webAdminV4 = 0;
+            }
+
+            if (!webAdminV4 || !webAdminV4->isSane())
             {
                CritLog(<<"Failed to start WebAdminV4");
+               delete webAdminV4;
                cleanupObjects();
                return false;
             }
 
-            mWebAdminList->push_back(WebAdminV4);
+            mWebAdminList.push_back(webAdminV4);
          }
 
          if(mUseV6 && DnsUtil::isIpV6Address(*it)) 
          {
-            WebAdmin* WebAdminV6 = new WebAdmin(*mProxy,
-                                                *mRegistrationPersistenceManager, 
-                                                mHttpRealm, 
-                                                httpPort,
-                                                V6,
-                                                *it);
+            WebAdmin* webAdminV6 = 0;
 
-            if (!WebAdminV6->isSane())
+            try 
+            {
+               webAdminV6 = new WebAdmin(*mProxy,
+                                         *mRegistrationPersistenceManager, 
+                                         mHttpRealm, 
+                                         httpPort,
+                                         V6,
+                                         *it);
+            } 
+            catch(WebAdmin::ConfigException& ex) 
+            {
+               ErrLog(<<"Exception when starting WebAdmin: " << ex.getMessage());
+               webAdminV6 = 0;
+            }
+
+            if (!webAdminV6 || !webAdminV6->isSane())
             {
                CritLog(<<"Failed to start WebAdminV6");
+               delete webAdminV6;
                cleanupObjects();
                return false;
             }
 
-            mWebAdminList->push_back(WebAdminV6);
+            mWebAdminList.push_back(webAdminV6);
          }
       }
 
       // This shouldn't happen because it would return false before
       // it reached this point
-      if(!mWebAdminList->empty())
+      if(!mWebAdminList.empty())
       {
-         mWebAdminThread = new WebAdminThread(*mWebAdminList);
+         mWebAdminThread = new WebAdminThread(mWebAdminList);
          return true;
       }
    }
@@ -997,14 +1187,13 @@ ReproRunner::createRegSync()
 void
 ReproRunner::createCommandServer()
 {
-   assert(!mCommandServerList);
+   assert(mCommandServerList.empty());
    assert(!mCommandServerThread);
 
    std::vector<resip::Data> commandServerBindAddresses;
    mProxyConfig->getConfigValue("CommandBindAddress", commandServerBindAddresses);
    int commandPort = mProxyConfig->getConfigInt("CommandPort", 5081);
 
-   mCommandServerList = new std::list<CommandServer*>;
    if(commandPort != 0)
    {
       if(commandServerBindAddresses.empty())
@@ -1023,36 +1212,38 @@ ReproRunner::createCommandServer()
       {
          if(mUseV4 && DnsUtil::isIpV4Address(*it))
          {
-            CommandServer* CommandServerV4 = new CommandServer(*this, *it, commandPort, V4);
+            CommandServer* pCommandServerV4 = new CommandServer(*this, *it, commandPort, V4);
 
-            if(CommandServerV4->isSane())
+            if(pCommandServerV4->isSane())
             {
-               mCommandServerList->push_back(CommandServerV4);
+               mCommandServerList.push_back(pCommandServerV4);
             }
             else
             {
                CritLog(<<"Failed to start CommandServerV4");
+               delete pCommandServerV4;
             }
          }
 
          if(mUseV6 && DnsUtil::isIpV6Address(*it))
          {
-            CommandServer* CommandServerV6 = new CommandServer(*this, *it, commandPort, V6);
+            CommandServer* pCommandServerV6 = new CommandServer(*this, *it, commandPort, V6);
 
-            if(CommandServerV6->isSane())
+            if(pCommandServerV6->isSane())
             {
-               mCommandServerList->push_back(CommandServerV6);
+               mCommandServerList.push_back(pCommandServerV6);
             }
             else
             {
                CritLog(<<"Failed to start CommandServerV6");
+               delete pCommandServerV6;
             }
          }
       }
 
-      if(!mCommandServerList->empty())
+      if(!mCommandServerList.empty())
       {
-         mCommandServerThread = new CommandServerThread(*mCommandServerList);
+         mCommandServerThread = new CommandServerThread(mCommandServerList);
       }
    }
 }
@@ -1119,7 +1310,9 @@ ReproRunner::addDomains(TransactionUser& tu, bool log)
    tu.addDomain("127.0.0.1"); */
 
    if( realm.empty() )
+   {
       realm = "Unconfigured";
+   }
 
    return realm;
 }
@@ -1131,14 +1324,27 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
    assert(mSipStack);
    allTransportsSpecifyRecordRoute=false;
    bool useEmailAsSIP = mProxyConfig->getConfigBool("TLSUseEmailAsSIP", false);
+
+   Data wsCookieAuthSharedSecret = mProxyConfig->getConfigData("WSCookieAuthSharedSecret", "");
+   SharedPtr<BasicWsConnectionValidator> basicWsConnectionValidator; // NULL
+   SharedPtr<WsCookieContextFactory> wsCookieContextFactory;
+   if(!wsCookieAuthSharedSecret.empty())
+   {
+      basicWsConnectionValidator.reset(new BasicWsConnectionValidator(wsCookieAuthSharedSecret));
+      Data infoCookieName = mProxyConfig->getConfigData("WSCookieNameInfo", "");
+      Data extraCookieName = mProxyConfig->getConfigData("WSCookieNameExtra", "");
+      Data macCookieName = mProxyConfig->getConfigData("WSCookieNameMac", "");
+
+      wsCookieContextFactory.reset(new BasicWsCookieContextFactory(infoCookieName, extraCookieName, macCookieName));
+   }
+
    try
    {
       // Check if advanced transport settings are provided
-      unsigned int transportNum = 1;
-      Data settingKeyBase("Transport" + Data(transportNum));
-      Data interfaceSettingKey(settingKeyBase + "Interface");
-      Data interfaceSettings = mProxyConfig->getConfigData(interfaceSettingKey, "", true);
-      if(!interfaceSettings.empty())
+      std::set<Data> interfaceKeys;
+      mProxyConfig->getConfigIndexKeys("Transport", interfaceKeys);
+      DebugLog(<<"Found " << interfaceKeys.size() << " interface(s) defined in the advanced format");
+      if(!interfaceKeys.empty())
       {
          // Sample config file format for advanced transport settings
          // Transport1Interface = 192.168.1.106:5061
@@ -1153,8 +1359,14 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
          allTransportsSpecifyRecordRoute = true;
 
          const char *anchor;
-         while(!interfaceSettings.empty())
+         for(std::set<Data>::iterator it = interfaceKeys.begin();
+            it != interfaceKeys.end();
+            it++)
          {
+            const Data& settingKeyBase = *it;
+            DebugLog(<< "checking values for transport: " << settingKeyBase);
+            Data interfaceSettingKey(settingKeyBase + "Interface");
+            Data interfaceSettings = mProxyConfig->getConfigData(interfaceSettingKey, "", true);
             Data typeSettingKey(settingKeyBase + "Type");
             Data tlsDomainSettingKey(settingKeyBase + "TlsDomain");
             Data tlsCertificateSettingKey(settingKeyBase + "TlsCertificate");
@@ -1210,7 +1422,7 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
                   CritLog(<< "Unknown TLS client verification mode found in " << tlsCVMSettingKey << " setting: " << tlsCVMValue);
                }
 
-#if defined(USE_SSL)
+#ifdef USE_SSL
                // Make sure certificate material available before trying to instantiate Transport
                if(isSecure(tt))
                {
@@ -1238,8 +1450,10 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
                                  Data::Empty,  // private key passphrase - not currently used
                                  SecurityTypes::TLSv1, // sslType
                                  0,            // transport flags
+                                 tlsCertificate, tlsPrivateKey,
                                  cvm,          // tls client verification mode
-                                 useEmailAsSIP);
+                                 useEmailAsSIP,
+                                 basicWsConnectionValidator, wsCookieContextFactory);
 
                if (t && rcvBufLen>0 )
                {
@@ -1259,7 +1473,7 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
                   {
                      if(isEqualNoCase(recordRouteUri, "auto")) // auto generated record route uri
                      {
-                        if(tt == TLS || tt == DTLS)
+                        if(isSecure(tt))
                         {
                            NameAddr rr;
                            rr.uri().host()=tlsDomain;
@@ -1301,12 +1515,6 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
                CritLog(<< "Port not specified in " << interfaceSettingKey << " setting: expected format is <IPAddress>:<Port>");
                return false;
             }
-
-            // Check if there is another transport
-            transportNum++;
-            settingKeyBase = Data("Transport" + Data(transportNum));
-            interfaceSettingKey = Data(settingKeyBase + "Interface");
-            interfaceSettings = mProxyConfig->getConfigData(interfaceSettingKey, "", true);
          }
       }
       else
@@ -1314,6 +1522,8 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
          int udpPort = mProxyConfig->getConfigInt("UDPPort", 5060);
          int tcpPort = mProxyConfig->getConfigInt("TCPPort", 5060);
          int tlsPort = mProxyConfig->getConfigInt("TLSPort", 5061);
+         int wsPort = mProxyConfig->getConfigInt("WSPort", 80);
+         int wssPort = mProxyConfig->getConfigInt("WSSPort", 443);
          int dtlsPort = mProxyConfig->getConfigInt("DTLSPort", 0);
          Data tlsDomain = mProxyConfig->getConfigData("TLSDomainName", "");
          Data tlsCertificate = mProxyConfig->getConfigData("TLSCertificate", "");
@@ -1333,9 +1543,9 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
             CritLog(<< "Unknown TLS client verification mode found in TLSClientVerification setting: " << tlsCVMValue);
          }
 
-#if defined(USE_SSL)
+#ifdef USE_SSL
          // Make sure certificate material available before trying to instantiate Transport
-         if (tlsPort || dtlsPort)
+         if (tlsPort || wssPort || dtlsPort)
          {
             Security* security = mSipStack->getSecurity();
             assert(security != 0);
@@ -1366,13 +1576,23 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
          }
          if (tlsPort)
          {
-            if (mUseV4) mSipStack->addTransport(TLS, tlsPort, V4, StunEnabled, Data::Empty, tlsDomain, Data::Empty, SecurityTypes::TLSv1, 0, cvm, useEmailAsSIP);
-            if (mUseV6) mSipStack->addTransport(TLS, tlsPort, V6, StunEnabled, Data::Empty, tlsDomain, Data::Empty, SecurityTypes::TLSv1, 0, cvm, useEmailAsSIP);
+            if (mUseV4) mSipStack->addTransport(TLS, tlsPort, V4, StunEnabled, Data::Empty, tlsDomain, Data::Empty, SecurityTypes::TLSv1, 0, tlsCertificate, tlsPrivateKey, cvm, useEmailAsSIP);
+            if (mUseV6) mSipStack->addTransport(TLS, tlsPort, V6, StunEnabled, Data::Empty, tlsDomain, Data::Empty, SecurityTypes::TLSv1, 0, tlsCertificate, tlsPrivateKey, cvm, useEmailAsSIP);
+         }
+         if (wsPort)
+         {
+            if (mUseV4) mSipStack->addTransport(WS, wsPort, V4, StunEnabled,  Data::Empty, Data::Empty, Data::Empty, SecurityTypes::NoSSL, 0, "", "", SecurityTypes::None, false, basicWsConnectionValidator, wsCookieContextFactory);
+            if (mUseV6) mSipStack->addTransport(WS, wsPort, V6, StunEnabled,  Data::Empty, Data::Empty, Data::Empty, SecurityTypes::NoSSL, 0, "", "", SecurityTypes::None, false, basicWsConnectionValidator, wsCookieContextFactory);
+         }
+         if (wssPort)
+         {
+            if (mUseV4) mSipStack->addTransport(WSS, wssPort, V4, StunEnabled, Data::Empty, tlsDomain, Data::Empty, SecurityTypes::TLSv1, 0, tlsCertificate, tlsPrivateKey, cvm, useEmailAsSIP, basicWsConnectionValidator, wsCookieContextFactory);
+            if (mUseV6) mSipStack->addTransport(WSS, wssPort, V6, StunEnabled, Data::Empty, tlsDomain, Data::Empty, SecurityTypes::TLSv1, 0, tlsCertificate, tlsPrivateKey, cvm, useEmailAsSIP, basicWsConnectionValidator, wsCookieContextFactory);
          }
          if (dtlsPort)
          {
-            if (mUseV4) mSipStack->addTransport(DTLS, dtlsPort, V4, StunEnabled, Data::Empty, tlsDomain);
-            if (mUseV6) mSipStack->addTransport(DTLS, dtlsPort, V6, StunEnabled, Data::Empty, tlsDomain);
+            if (mUseV4) mSipStack->addTransport(DTLS, dtlsPort, V4, StunEnabled, Data::Empty, tlsDomain, Data::Empty, SecurityTypes::TLSv1, 0, tlsCertificate, tlsPrivateKey);
+            if (mUseV6) mSipStack->addTransport(DTLS, dtlsPort, V6, StunEnabled, Data::Empty, tlsDomain, Data::Empty, SecurityTypes::TLSv1, 0, tlsCertificate, tlsPrivateKey);
          }
       }
    }
@@ -1391,67 +1611,6 @@ ReproRunner::addProcessor(repro::ProcessorChain& chain, std::auto_ptr<Processor>
    chain.addProcessor(processor);
 }
 
-void
-ReproRunner::loadCommonNameMappings()
-{
-   // Already loaded?
-   if(!mCommonNameMappings.empty())
-      return;
-
-   Data mappingsFileName = mProxyConfig->getConfigData("CommonNameMappings", "");
-   if(mappingsFileName.empty())
-      return;
-
-   InfoLog(<< "trying to load common name mappings from file: " << mappingsFileName);
-
-   ifstream mappingsFile(mappingsFileName.c_str());
-   if(!mappingsFile)
-   {
-      ErrLog(<< "failed to open mappings file: " << mappingsFileName << ", aborting");
-      throw std::runtime_error("Error opening/reading mappings file");
-   }
-
-   string sline;
-   while(getline(mappingsFile, sline))
-   {
-      Data line(sline);
-      Data cn;
-      PermittedFromAddresses permitted;
-      ParseBuffer pb(line);
-
-      pb.skipWhitespace();
-      const char * anchor = pb.position();
-      if(pb.eof() || *anchor == '#') continue;  // if line is a comment or blank then skip it
-
-      // Look for end of name
-      pb.skipToOneOf("\t");
-      pb.data(cn, anchor);
-      pb.skipChar('\t');
-
-      while(!pb.eof())
-      {
-         pb.skipWhitespace();
-         if(pb.eof())
-            continue;
-
-         Data value;
-         anchor = pb.position();
-         pb.skipToOneOf(",\r\n ");
-         pb.data(value, anchor);
-         if(!value.empty())
-         {
-            StackLog(<< "Loading CN '" << cn << "', found mapping '" << value << "'");
-            permitted.insert(value);
-         }
-         if(!pb.eof())
-            pb.skipChar();
-      }
-
-      DebugLog(<< "Loaded mapping for CN '" << cn << "', " << permitted.size() << " mapping(s)");
-      mCommonNameMappings[cn] = permitted;
-   }
-}
-
 void  // Monkeys
 ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
 {
@@ -1465,25 +1624,28 @@ ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
    addProcessor(chain, std::auto_ptr<Processor>(new IsTrustedNode(*mProxyConfig)));
 
    // Add Certificate Authenticator - if required
-   if(mProxyConfig->getConfigBool("EnableCertificateAuthenticator", false))
+   assert(mAuthFactory);
+   if(mAuthFactory->certificateAuthEnabled())
    {
       // TODO: perhaps this should be initialised from the trusted node
       // monkey?  Or should the list of trusted TLS peers be independent
       // from the trusted node list?
       // Should we used the same trustedPeers object that was
       // passed to TlsPeerAuthManager perhaps?
-      std::set<Data> trustedPeers;
-      loadCommonNameMappings();
-      addProcessor(chain, std::auto_ptr<Processor>(new CertificateAuthenticator(*mProxyConfig, mSipStack, trustedPeers, true, mCommonNameMappings)));
+      addProcessor(chain, mAuthFactory->getCertificateAuthenticator());
+   }
+
+   Data wsCookieAuthSharedSecret = mProxyConfig->getConfigData("WSCookieAuthSharedSecret", "");
+   Data wsCookieExtraHeaderName = mProxyConfig->getConfigData("WSCookieExtraHeaderName", "X-WS-Session-Extra");
+   if(!mAuthFactory->digestAuthEnabled() && !wsCookieAuthSharedSecret.empty())
+   {
+      addProcessor(chain, std::auto_ptr<Processor>(new CookieAuthenticator(wsCookieAuthSharedSecret, wsCookieExtraHeaderName, mSipStack)));
    }
 
    // Add digest authenticator monkey - if required
-   if (!mSipAuthDisabled)
+   if (mAuthFactory->digestAuthEnabled())
    {
-      assert(mAuthRequestDispatcher);
-      DigestAuthenticator* da = new DigestAuthenticator(*mProxyConfig, mAuthRequestDispatcher);
-
-      addProcessor(chain, std::auto_ptr<Processor>(da)); 
+      addProcessor(chain, mAuthFactory->getDigestAuthenticator()); 
    }
 
    // Add am I responsible monkey
@@ -1521,7 +1683,7 @@ ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
    }
 
    // Add location server monkey
-   addProcessor(chain, std::auto_ptr<Processor>(new LocationServer(*mProxyConfig, *mRegistrationPersistenceManager, mAuthRequestDispatcher)));
+   addProcessor(chain, std::auto_ptr<Processor>(new LocationServer(*mProxyConfig, *mRegistrationPersistenceManager, mAuthFactory->getDispatcher())));
 
    // Add message silo monkey
    if(mProxyConfig->getConfigBool("MessageSiloEnabled", false))
@@ -1577,6 +1739,16 @@ ReproRunner::makeTargetProcessorChain(ProcessorChain& chain)
    addProcessor(chain, std::auto_ptr<Processor>(new SimpleTargetHandler)); 
 }
 
+bool 
+ReproRunner::operator()(resip::StatisticsMessage &statsMessage)
+{
+   // Dispatch to each command server
+   for(std::list<CommandServer*>::iterator it = mCommandServerList.begin(); it != mCommandServerList.end(); it++)
+   {
+       (*it)->handleStatisticsMessage(statsMessage);
+   }
+   return true;
+}
 
 /* ====================================================================
  * The Vovida Software License, Version 1.0 
