@@ -46,6 +46,8 @@ RequestContext::RequestContext(Proxy& proxy,
    mProxy(proxy),
    mResponseContext(*this),
    mTCSerial(0),
+   mSessionCreatedEventSent(false),
+   mSessionEstablishedEventSent(false),
    mKeyValueStore(*Proxy::getRequestKeyValueStoreKeyAllocator())
 {
    mInitialTimerCSet=false;
@@ -149,6 +151,21 @@ RequestContext::process(std::auto_ptr<resip::SipMessage> sipMessage)
    {
       DebugLog(<<"Got a request.");
       bool postProcess=false;
+
+      Uri& requestUri = sip->header(h_RequestLine).uri();
+      if(requestUri.exists(resip::p_wsSrcIp) &&
+            requestUri.exists(resip::p_wsSrcPort) &&
+            !isWebSocket(sip->getSource().getType()))
+      {
+         requestUri.host() = requestUri.param(resip::p_wsSrcIp);
+         requestUri.remove(resip::p_wsSrcIp);
+         requestUri.port() = requestUri.param(resip::p_wsSrcPort);
+         requestUri.remove(resip::p_wsSrcPort);
+         requestUri.param(resip::p_transport) = "WS";
+         DebugLog(<< "recognised request for WS peer, setting forceTarget to " << requestUri);
+         sip->setForceTarget(requestUri);
+      }
+
       switch(mOriginalRequest->method())
       {
          case ACK:
@@ -220,6 +237,10 @@ RequestContext::processRequestInviteTransaction(SipMessage* msg, bool original)
    {
       if(msg->method()==CANCEL)
       {
+         if(mSessionCreatedEventSent && !mSessionEstablishedEventSent)
+         {
+            getProxy().doSessionAccounting(*msg, true /* received */, *this);
+         }
          mResponseContext.processCancel(*msg);
          doPostProcess=true;
       }
@@ -250,17 +271,9 @@ RequestContext::processRequestInviteTransaction(SipMessage* msg, bool original)
             InfoLog(<<"Got an ACK within an INVITE transaction, but our "
                      "response was a 2xx. Someone didn't change their tid "
                      "like they were supposed to...");
-            if(
-               (
-                  msg->exists(h_Routes) && 
-                  !msg->header(h_Routes).empty()
-               ) 
-               ||
-               (
-                  !getProxy().isMyUri(msg->header(h_RequestLine).uri()) && 
-                  (msg->header(h_From).isWellFormed() && getProxy().isMyUri(msg->header(h_From).uri())) 
-               )
-               )
+            if((msg->exists(h_Routes) && !msg->header(h_Routes).empty()) ||   // If ACK/200 has a Route header   OR
+               (!getProxy().isMyUri(msg->header(h_RequestLine).uri()) &&      // RequestUri is not us and From Uri is our domain
+                (msg->header(h_From).isWellFormed() && getProxy().isMyUri(msg->header(h_From).uri()))))
             {
                forwardAck200(*msg);
             }
@@ -375,7 +388,7 @@ RequestContext::processRequestAckTransaction(SipMessage* msg, bool original)
          handleSelfAimedStrayAck(msg);
       }
       // Note: mTopRoute is only populated if RemoveTopRouteIfSelf successfully removes the top route.
-      else if(!mTopRoute.uri().host().empty() || getProxy().isMyUri(msg->header(h_From).uri()))
+      else if(msg->hasForceTarget() || !mTopRoute.uri().host().empty() || getProxy().isMyUri(msg->header(h_From).uri()))
       {
          // Top most route is us, or From header uri is ours.  Note:  The From check is 
          // required to interoperate with endpoints that configure outbound proxy 
@@ -623,8 +636,17 @@ RequestContext::doPostResponseProcessing(SipMessage* msg)
          << "a sip response (_not_ a NIT/408): all transactions are terminated,"
          << " but we have not sent a final response. (What happened here?) ");
 
-         // Send best response
-         mResponseContext.forwardBestResponse();
+         // Send best response if there is one - otherwise send 500
+         if(mResponseContext.mBestResponse.isResponse())
+         {
+            mResponseContext.forwardBestResponse();
+         }
+         else
+         {
+            resip::SipMessage response;
+            Helper::makeResponse(response, *mOriginalRequest, 500); 
+            sendResponse(response);
+         }
       }
    }
 }
@@ -962,6 +984,10 @@ RequestContext::sendResponse(SipMessage& msg)
       {
          msg.header(h_Server).value() = serverText;
       }
+      if(mSessionCreatedEventSent && !mSessionEstablishedEventSent)
+      {
+         getProxy().doSessionAccounting(msg, false /* received */, *this);
+      }
       send(msg);
    }
 }
@@ -1040,6 +1066,41 @@ RequestContext::getTopRoute()
    return mTopRoute;
 }
 
+const resip::Data&
+RequestContext::getDigestRealm()
+{
+   // (1) Check Preferred Identity
+   if (mOriginalRequest->exists(h_PPreferredIdentities))
+   {
+      // !abr! Add this when we get a chance
+      // find the fist sip or sips P-Preferred-Identity header
+      // for (;;)
+      // {
+      //    if ((i->uri().scheme() == Symbols::SIP) || (i->uri().scheme() == Symbols::SIPS))
+      //    {
+      //       return i->uri().host();
+      //    }
+      // }
+   }
+
+   // (2) Check From domain
+   if (mProxy.isMyDomain(mOriginalRequest->header(h_From).uri().host()))
+   {
+      return mOriginalRequest->header(h_From).uri().host();
+   }
+
+   // (3) Check Top Route Header
+   if (mOriginalRequest->exists(h_Routes) &&
+         mOriginalRequest->header(h_Routes).size()!=0 &&
+         mOriginalRequest->header(h_Routes).front().isWellFormed())
+   {
+      // !abr! Add this when we get a chance
+   }
+
+   // (4) Punt: Use Request URI
+   return mOriginalRequest->header(h_RequestLine).uri().host();
+}
+
 EncodeStream&
 repro::operator<<(EncodeStream& strm, const RequestContext& rc)
 {
