@@ -14,6 +14,7 @@
 #include "rutil/Inserter.hxx"
 #include "rutil/Random.hxx"
 #include "rutil/ParseBuffer.hxx"
+#include "rutil/TransportType.hxx"
 #include "rutil/WinLeakCheck.hxx"
 
 #define RESIPROCATE_SUBSYSTEM Subsystem::DUM
@@ -35,17 +36,25 @@ ClientRegistration::ClientRegistration(DialogUsageManager& dum,
      mLastRequest(request),
      mTimerSeq(0),
      mState(mLastRequest->exists(h_Contacts) ? Adding : Querying),
+     mEnding(false),
      mEndWhenDone(false),
      mUserRefresh(false),
      mRegistrationTime(mDialogSet.mUserProfile->getDefaultRegistrationTime()),
      mExpires(0),
+     mRefreshTime(0),
      mQueuedState(None),
      mQueuedRequest(new SipMessage)
 {
    // If no Contacts header, this is a query
    if (mLastRequest->exists(h_Contacts))
    {
-      mMyContacts = mLastRequest->header(h_Contacts);
+      NameAddr all;
+      all.setAllContacts();
+      if(!(mLastRequest->header(h_Contacts).front() == all))
+      {
+         // store if not special all contacts header
+         mMyContacts = mLastRequest->header(h_Contacts);
+      }
    }
 
    if(mLastRequest->exists(h_Expires) && 
@@ -219,6 +228,13 @@ ClientRegistration::removeMyBindings(bool stopRegisteringWhenDone)
 
    if (mQueuedState == None)
    {
+      if(mEnding && whenExpires() == 0)
+      {
+         assert(mEndWhenDone);  // will always be true when mEnding is true
+         // We are not actually registered, and we are ending - no need to send un-register - just terminate now
+         stopRegistering();
+         return;
+      }
       send(next);
    }
 }
@@ -313,16 +329,25 @@ ClientRegistration::allContacts()
 UInt32
 ClientRegistration::whenExpires() const
 {
-// !cj! - TODO - I'm supisious these time are getting confused on what units they are in 
    UInt64 now = Timer::getTimeSecs();
-   UInt64 ret = mExpires - now;
-   return (UInt32)ret;
+   if(mExpires > now)
+   {
+       return (UInt32)(mExpires - now);
+   }
+   else
+   {
+       return 0;
+   }
 }
 
 void
 ClientRegistration::end()
 {
-   removeMyBindings(true);
+   if(!mEnding)
+   {
+      mEnding = true;
+      removeMyBindings(true);
+   }
 }
 
 class ClientRegistrationEndCommand : public DumCommandAdapter
@@ -331,7 +356,6 @@ public:
    ClientRegistrationEndCommand(const ClientRegistrationHandle& clientRegistrationHandle)
       : mClientRegistrationHandle(clientRegistrationHandle)
    {
-
    }
 
    virtual void executeCommand()
@@ -403,12 +427,11 @@ ClientRegistration::dispatch(const SipMessage& msg)
 
       if(msg.isExternal())
       {
-         const Data& receivedTransport = msg.header(h_Vias).front().transport();
+         resip::TransportType receivedTransport = toTransportType(
+            msg.header(h_Vias).front().transport());
          if(keepAliveTime == 0)
          {
-            if(receivedTransport == Symbols::TCP ||
-               receivedTransport == Symbols::TLS ||
-               receivedTransport == Symbols::SCTP)
+            if(isReliable(receivedTransport))
             {
                keepAliveTime = mDialogSet.mUserProfile->getKeepAliveTimeForStream();
             }
@@ -481,7 +504,9 @@ ClientRegistration::dispatch(const SipMessage& msg)
          // !ah! take list of ctcs and push into mMy or mOther as required.
 
          // make timers to re-register
+         UInt64 nowSecs = Timer::getTimeSecs();
          UInt32 expiry = calculateExpiry(msg);
+         mExpires = nowSecs + expiry;
          if(msg.exists(h_Contacts))
          {
             mAllContacts = msg.header(h_Contacts);
@@ -496,7 +521,7 @@ ClientRegistration::dispatch(const SipMessage& msg)
             if(expiry >= UnreasonablyLowExpirationThreshold)
             {
                int exp = Helper::aBitSmallerThan(expiry);
-               mExpires = exp + Timer::getTimeSecs();
+               mRefreshTime = exp + nowSecs;
                mDum.addTimer(DumTimeout::Registration,
                              exp,
                              getBaseHandle(),
@@ -567,6 +592,13 @@ ClientRegistration::dispatch(const SipMessage& msg)
 
          if (mQueuedState != None)
          {
+            if(mQueuedState == Removing && mEnding && whenExpires() == 0)
+            {
+               assert(mEndWhenDone);  // will always be true when mEnding is true
+               // We are not actually registered, and we are ending - no need to send un-register - just terminate now
+               stopRegistering();
+               return;
+            }
             InfoLog (<< "Sending queued request: " << *mQueuedRequest);
             mState = mQueuedState;
             mQueuedState = None;
@@ -611,7 +643,7 @@ ClientRegistration::dispatch(const SipMessage& msg)
                else
                {
                   DebugLog(<< "Application requested delayed retry on 408 or internal 503: " << retry);
-                  mExpires = 0;
+                  mRefreshTime = 0;
                   switch(mState)
                   {
                   case Adding:
@@ -894,7 +926,7 @@ ClientRegistration::checkProfileRetry(const SipMessage& msg)
          // Use retry interval from error response
          retryInterval = msg.header(h_RetryAfter).value();
       }
-      mExpires = 0;
+      mRefreshTime = 0;
       switch(mState)
       {
       case Adding:
