@@ -520,7 +520,7 @@ DialogSet::dispatch(const SipMessage& msg)
          }
          else
          {
-          StackLog (<< "Matching dialog is destroying, dropping response message " << endl << msg);
+            StackLog (<< "Matching dialog is destroying, dropping response message " << endl << msg);
          }
          return;
       }
@@ -547,6 +547,7 @@ DialogSet::dispatch(const SipMessage& msg)
          case BYE:
          case INFO:
          case ACK:
+         case PRACK:
          case UPDATE:
             if(!dialog)
             {
@@ -742,7 +743,7 @@ DialogSet::dispatch(const SipMessage& msg)
             return;
 
          case MESSAGE:
-             if (dialog)
+            if (dialog)
             {
                 break;
             }
@@ -754,6 +755,7 @@ DialogSet::dispatch(const SipMessage& msg)
 
          case INFO:   
          case UPDATE:
+         case PRACK:
             if (dialog)
             {
                break;
@@ -791,7 +793,19 @@ DialogSet::dispatch(const SipMessage& msg)
    {
       if (msg.isRequest() && msg.header(h_RequestLine).method() == CANCEL)
       {
-         dispatchToAllDialogs(msg);
+         if(!mDialogs.empty())
+         {
+            dispatchToAllDialogs(msg);
+         }
+         else
+         {
+            DebugLog ( << "In DialogSet::dispatch, CANCEL received but no dialogs to dispatch to - respond with 481, msg: " << msg );
+            // Race condition - Dialogset is still around, but dialog is gone (potentially BYE'd).
+            // Need to respond to CANCEL in order for transaction in stack to go away
+            SharedPtr<SipMessage> response(new SipMessage);         
+            mDum.makeResponse(*response, msg, 481);
+            mDum.send(response);
+         }
          return;
       }
 
