@@ -1564,8 +1564,16 @@ DialogUsageManager::incomingProcess(std::auto_ptr<Message> msg)
          }
          else
          {
-            assert(dynamic_cast<SipMessage*>(msg.get()));
-            it = mIncomingFeatureChainMap.insert(lb, FeatureChainMap::value_type(tid, new DumFeatureChain(*this, mIncomingFeatureList, *mIncomingTarget)));
+            if(dynamic_cast<SipMessage*>(msg.get()))
+            {
+               it = mIncomingFeatureChainMap.insert(lb, FeatureChainMap::value_type(tid, new DumFeatureChain(*this, mIncomingFeatureList, *mIncomingTarget)));
+            }
+            else
+            {
+               // Certain messages from the wire (ie: CANCEL) can end a feature, however there may still be some 
+               // pending Async requests (non-SipMessages) that are coming in - just drop them if so
+               return;
+            }
          }
       }
       
@@ -1621,7 +1629,7 @@ DialogUsageManager::incomingProcess(std::auto_ptr<Message> msg)
                   DebugLog (<< "Failed required options validation " << *sipMsg);
                   return;
                }
-               if( !validate100RelSuport(*sipMsg) )
+               if( !validate100RelSupport(*sipMsg) )
                {
                   DebugLog (<< "Remote party does not support 100rel " << *sipMsg);
                   return;
@@ -1792,7 +1800,7 @@ DialogUsageManager::validateRequiredOptions(const SipMessage& request)
 
 
 bool
-DialogUsageManager::validate100RelSuport(const SipMessage& request)
+DialogUsageManager::validate100RelSupport(const SipMessage& request)
 {
    if(request.header(h_RequestLine).getMethod() == INVITE)
    {
@@ -1807,7 +1815,9 @@ DialogUsageManager::validate100RelSuport(const SipMessage& request)
             sendResponse(failure);
       
             if(mRequestValidationHandler)
+            {
                mRequestValidationHandler->on100RelNotSupportedByRemote(request);
+            }
 
             return false;
          }
@@ -2039,7 +2049,7 @@ DialogUsageManager::processRequest(const SipMessage& request)
             }
             else
             {
-               InfoLog (<< "Received a CANCEL on a non-existent transaction ");
+               InfoLog (<< "Received a CANCEL on a non-existent transaction: tid=" << request.getTransactionId());
                SipMessage failure;
                makeResponse(failure, request, 481);
                sendResponse(failure);
@@ -2139,7 +2149,7 @@ DialogUsageManager::processResponse(const SipMessage& response)
          DebugLog ( << "DialogUsageManager::processResponse: " << std::endl << std::endl << response.brief());
          ds->dispatch(response);
       }
-      else
+       else
       {
 		 InfoLog (<< "Throwing away stray response: " << std::endl << std::endl << response.brief());
       }
@@ -2243,6 +2253,10 @@ DialogUsageManager::checkEventPackage(const SipMessage& request)
    {
       SharedPtr<SipMessage> response(new SipMessage);
       makeResponse(*response, request, failureCode);
+      if(failureCode == 489)
+      {
+         response->header(h_AllowEvents) = getMasterProfile()->getAllowedEvents();
+      }
       send(response);
       return false;
    }
@@ -2479,6 +2493,30 @@ DialogUsageManager::createDialogEventStateManager(DialogEventHandler* handler)
    return mDialogEventStateManager;
 }
 
+void 
+DialogUsageManager::setAdvertisedCapabilities(SipMessage& msg, SharedPtr<UserProfile> userProfile)
+{
+   if(userProfile->isAdvertisedCapability(Headers::Allow)) 
+   {
+      msg.header(h_Allows) = getMasterProfile()->getAllowedMethods();
+   }
+   if(userProfile->isAdvertisedCapability(Headers::AcceptEncoding)) 
+   {
+      msg.header(h_AcceptEncodings) = getMasterProfile()->getSupportedEncodings();
+   }
+   if(userProfile->isAdvertisedCapability(Headers::AcceptLanguage)) 
+   {
+      msg.header(h_AcceptLanguages) = getMasterProfile()->getSupportedLanguages();
+   }
+   if(userProfile->isAdvertisedCapability(Headers::AllowEvents)) 
+   {
+      msg.header(h_AllowEvents) = getMasterProfile()->getAllowedEvents();
+   }
+   if(userProfile->isAdvertisedCapability(Headers::Supported)) 
+   {
+      msg.header(h_Supporteds) = getMasterProfile()->getSupportedOptionTags();
+   }
+}
 
 /* ====================================================================
  * The Vovida Software License, Version 1.0
