@@ -38,6 +38,7 @@ Profile::reset()
    unsetDefaultSessionTime(); 
    unsetDefaultSessionTimerMode();   
    unset1xxRetransmissionTime();   
+   unset1xxRelResubmitTime();
    unsetOverrideHostAndPort(); 
    unsetAdvertisedCapabilities();
    unsetOutboundProxy(); 
@@ -377,6 +378,38 @@ Profile::unset1xxRetransmissionTime()
    }
 }
 
+void
+Profile::set1xxRelResubmitTime(int secs)
+{
+   m1xxRelResubmitTime = secs;
+   mHas1xxRelResubmitTime = true;
+}
+
+int 
+Profile::get1xxRelResubmitTime() const
+{
+   // Fall through seting (if required)
+   if(!mHas1xxRelResubmitTime && mBaseProfile.get())
+   {
+       return mBaseProfile->get1xxRelResubmitTime();
+   }
+   return m1xxRelResubmitTime;
+}
+
+void
+Profile::unset1xxRelResubmitTime()
+{
+   if(mBaseProfile.get()) 
+   {
+      mHas1xxRelResubmitTime = false;
+   }
+   else // No Base profile - so return to default setting
+   {
+      mHas1xxRelResubmitTime = true;
+      m1xxRelResubmitTime = 150;  // RFC3262 section says the UAS SHOULD send provisional reliable responses once every two and half minutes
+   }
+}
+
 void 
 Profile::setOverrideHostAndPort(const Uri& hostPort)
 {
