@@ -14,14 +14,14 @@
 using namespace resip;
 using namespace std;
 
-TlsPeerAuthManager::TlsPeerAuthManager(DialogUsageManager& dum, TargetCommand::Target& target, std::set<Data>& trustedPeers, bool thirdPartyRequiresCertificate) :
+TlsPeerAuthManager::TlsPeerAuthManager(DialogUsageManager& dum, TargetCommand::Target& target, const std::set<Data>& trustedPeers, bool thirdPartyRequiresCertificate) :
    DumFeature(dum, target),
    mTrustedPeers(trustedPeers),
    mThirdPartyRequiresCertificate(thirdPartyRequiresCertificate)
 {
 }
 
-TlsPeerAuthManager::TlsPeerAuthManager(DialogUsageManager& dum, TargetCommand::Target& target, std::set<Data>& trustedPeers, bool thirdPartyRequiresCertificate, CommonNameMappings& commonNameMappings) :
+TlsPeerAuthManager::TlsPeerAuthManager(DialogUsageManager& dum, TargetCommand::Target& target, const std::set<Data>& trustedPeers, bool thirdPartyRequiresCertificate, CommonNameMappings& commonNameMappings) :
    DumFeature(dum, target),
    mTrustedPeers(trustedPeers),
    mThirdPartyRequiresCertificate(thirdPartyRequiresCertificate),
@@ -72,11 +72,6 @@ TlsPeerAuthManager::authorizedForThisIdentity(
    for(; it != peerNames.end(); ++it)
    {
       const Data& i = *it;
-      if(mTrustedPeers.find(i) != mTrustedPeers.end())
-      {
-         DebugLog(<< "Matched certificate name " << i << " is a trusted peer, not checking against From URI");
-         return true;
-      }
       if(i == aor)
       {
          DebugLog(<< "Matched certificate name " << i << " against full AoR " << aor);
@@ -135,12 +130,17 @@ TlsPeerAuthManager::handle(SipMessage* sipMessage)
    }
 
    // We are only concerned with connections over TLS
-   if(!sipMessage->isExternal() || sipMessage->getSource().getType() != TLS)
+   if(!sipMessage->isExternal() || !isSecure(sipMessage->getSource().getType()))
    {
       DebugLog(<<"Can't validate certificate on non-TLS connection");
       return Skipped;
    }
 
+   if(isTrustedSource(*sipMessage))
+   {
+      return Authorized;
+   }
+
    const std::list<resip::Data> &peerNames = sipMessage->getTlsPeerNames();
    if (mDum.isMyDomain(sipMessage->header(h_From).uri().host()))
    {
@@ -194,6 +194,26 @@ TlsPeerAuthManager::requiresAuthorization(const SipMessage& msg)
    return true;
 }
 
+bool
+TlsPeerAuthManager::isTrustedSource(const SipMessage& msg)
+{
+   // over-ride this method to implement some other policy
+
+   const std::list<resip::Data> &peerNames = msg.getTlsPeerNames();
+   std::list<Data>::const_iterator it = peerNames.begin();
+   for(; it != peerNames.end(); ++it)
+   {
+      const Data& i = *it;
+      if(mTrustedPeers.find(i) != mTrustedPeers.end())
+      {
+         DebugLog(<< "Matched certificate name " << i << " is a trusted peer");
+         return true;
+      }
+   }
+
+   return false;
+}
+
 /* ====================================================================
  *
  * Copyright (c) 2012 Daniel Pocock  All rights reserved.
