@@ -1,4 +1,7 @@
 #include <asio.hpp>
+#ifdef USE_SSL
+#include <asio/ssl.hpp>
+#endif
 #include <boost/function.hpp>
 #include <rutil/Data.hxx>
 
