@@ -26,6 +26,7 @@ using namespace std;
 
 static const resip::ExtensionParameter p_localonly("local-only");
 static const resip::ExtensionParameter p_remoteonly("remote-only");
+static const resip::ExtensionParameter p_participantonly("participant-only");
 static const resip::ExtensionParameter p_repeat("repeat");
 static const resip::ExtensionParameter p_prefetch("prefetch");
 
@@ -182,9 +183,11 @@ MediaResourceParticipant::startPlay()
       case Tone:
       {
          int toneid;
+         bool isDtmf = false;
          if(mMediaUrl.host().size() == 1)
          {
             toneid = mMediaUrl.host().at(0);
+            isDtmf = true;
          }
          else
          {
@@ -204,7 +207,28 @@ MediaResourceParticipant::startPlay()
             }
          }
 
-         OsStatus status = getMediaInterface()->getInterface()->startTone(toneid, mRemoteOnly ? FALSE : TRUE /* local */, mLocalOnly ? FALSE : TRUE /* remote */);
+         OsStatus status = OS_FAILED;
+         if(mMediaUrl.exists(p_participantonly))
+         {
+            int partHandle = mMediaUrl.param(p_participantonly).convertInt();
+            RemoteParticipant* participant = dynamic_cast<RemoteParticipant*>(mConversationManager.getParticipant(partHandle));
+            if(participant)
+            {
+               StackLog(<<"sending tone to sipX connection: " << participant->getMediaConnectionId());
+               // this uses the original API, where both inband and RFC2833 tones are always sent simultaneously:
+               //status = getMediaInterface()->getInterface()->startChannelTone(participant->getMediaConnectionId(), toneid, mRemoteOnly ? FALSE : TRUE /* local */, mLocalOnly ? FALSE : TRUE /* remote */);
+               // this is for newer sipXtapi API, option to suppress inband tones:
+               status = getMediaInterface()->getInterface()->startChannelTone(participant->getMediaConnectionId(), toneid, mRemoteOnly ? FALSE : TRUE /* local */, mLocalOnly ? FALSE : TRUE /* remote */, !isDtmf /* inband */, true /* RFC 4733 */);
+            }
+            else
+            {
+               WarningLog(<<"Participant " << partHandle << " no longer exists or invalid");
+            }
+         }
+         else
+         {
+            status = getMediaInterface()->getInterface()->startTone(toneid, mRemoteOnly ? FALSE : TRUE /* local */, mLocalOnly ? FALSE : TRUE /* remote */);
+         }
          if(status == OS_SUCCESS)
          {
             mPlaying = true;
@@ -388,7 +412,28 @@ MediaResourceParticipant::destroyParticipant()
       {
       case Tone:
          {
-            OsStatus status = getMediaInterface()->getInterface()->stopTone();
+            OsStatus status = OS_FAILED;
+            if(mMediaUrl.exists(p_participantonly))
+            {
+               bool isDtmf = (mMediaUrl.host().size() == 1);
+               int partHandle = mMediaUrl.param(p_participantonly).convertInt();
+               RemoteParticipant* participant = dynamic_cast<RemoteParticipant*>(mConversationManager.getParticipant(partHandle));
+               if(participant)
+               {
+                  // this uses the original API, where both inband and RFC2833 tones are always sent simultaneously:
+                  //status = getMediaInterface()->getInterface()->stopChannelTone(participant->getMediaConnectionId());
+                  // this is for newer sipXtapi API, option to suppress inband tones:
+                  status = getMediaInterface()->getInterface()->stopChannelTone(participant->getMediaConnectionId(), !isDtmf, true);
+               }
+               else
+               {
+                  WarningLog(<<"Participant " << partHandle << " no longer exists or invalid");
+               }
+            }
+            else
+            {
+               status = getMediaInterface()->getInterface()->stopTone();
+            }
             if(status != OS_SUCCESS)
             {
                WarningLog(<< "MediaResourceParticipant::destroyParticipant error calling stopTone: " << status);
