@@ -2,16 +2,30 @@
 #include "config.h"
 #endif
 
+#include <memory>
+
 #include "rutil/Logger.hxx"
 #include "resip/stack/ConnectionBase.hxx"
+#include "resip/stack/WsConnectionBase.hxx"
 #include "resip/stack/SipMessage.hxx"
+#include "resip/stack/WsDecorator.hxx"
+#include "resip/stack/Cookie.hxx"
+#include "resip/stack/WsBaseTransport.hxx"
+#include "resip/stack/WsCookieContext.hxx"
+#include "resip/stack/WsCookieContextFactory.hxx"
+#include "resip/stack/Symbols.hxx"
 #include "rutil/WinLeakCheck.hxx"
+#include "rutil/SharedPtr.hxx"
+#include "rutil/Sha1.hxx"
 
 #ifdef USE_SSL
 #include "resip/stack/ssl/Security.hxx"
 #include "resip/stack/ssl/TlsConnection.hxx"
+#include "rutil/ssl/SHA1Stream.hxx"
 #endif
 
+#include "rutil/MD5Stream.hxx"
+
 #ifdef USE_SIGCOMP
 #include <osc/Stack.h>
 #include <osc/TcpStream.h>
@@ -26,6 +40,12 @@ using namespace resip;
 char 
 ConnectionBase::connectionStates[ConnectionBase::MAX][32] = { "NewMessage", "ReadingHeaders", "PartialBody" };
 
+#ifndef RESIP_SIP_MSG_MAX_BYTES
+#define RESIP_SIP_MSG_MAX_BYTES  10485760
+#endif
+
+size_t
+ConnectionBase::messageSizeMax = RESIP_SIP_MSG_MAX_BYTES;
 
 ConnectionBase::ConnectionBase(Transport* transport, const Tuple& who, Compression &compression)
    : mSendPos(0),
@@ -44,6 +64,7 @@ ConnectionBase::ConnectionBase(Transport* transport, const Tuple& who, Compressi
      mBuffer(0),
      mBufferPos(0),
      mBufferSize(0),
+     mWsFrameExtractor(messageSizeMax),
      mLastUsed(Timer::getTimeMs()),
      mConnState(NewMessage)
 {
@@ -209,7 +230,7 @@ ConnectionBase::preparseNewBytes(int bytesRead)
          unsigned int numUnprocessedChars = 
             (unsigned int)((mBuffer + chunkLength) - unprocessedCharPtr);
 
-         if(numUnprocessedChars > 2048 &&
+         if(numUnprocessedChars > ConnectionBase::ChunkSize &&
             scanChunkResult == MsgHeaderScanner::scrNextChunk)
          {
             WarningLog(<< "Discarding preparse; header-field-value (or "
@@ -321,12 +342,10 @@ ConnectionBase::preparseNewBytes(int bytesRead)
                return false;
             }
             
-            if(contentLength > 10485760 || contentLength < 0)
+            if(contentLength > messageSizeMax || contentLength < 0)
             {
-               // !bwc! No more than 10M, thanks. We should make this
-               // configurable.
-               WarningLog(<<"Absurdly large Content-Length in connection-based "
-                           "transport.");
+               WarningLog(<<"Content-Length in connection-based "
+                           "transport exceeds maximum " << messageSizeMax);
                delete mMessage;
                mMessage = 0;
                mBuffer = 0;
@@ -524,6 +543,326 @@ ConnectionBase::preparseNewBytes(int bytesRead)
    return true;
 }
 
+void
+ConnectionBase::wsParseCookies(CookieList& cookieList, const SipMessage* message)
+{
+   Data name;
+   Data value;
+   StringCategories::const_iterator it = message->header(h_Cookies).begin();
+   for (; it != message->header(h_Cookies).end(); ++it)
+   {
+      ParseBuffer pb((*it).value());
+      while(!pb.eof())
+      {
+         const char* anchor =  pb.skipWhitespace();
+
+         pb.skipToChar(Symbols::EQUALS[0]);
+         pb.data(name, anchor);
+
+         anchor = pb.skipChar(Symbols::EQUALS[0]);
+         if(*(pb.position()) == Symbols::DOUBLE_QUOTE[0])
+         {
+            anchor = pb.skipChar(Symbols::DOUBLE_QUOTE[0]);
+            pb.skipToChar(Symbols::DOUBLE_QUOTE[0]);
+            pb.data(value, anchor);
+            pb.skipChar(Symbols::DOUBLE_QUOTE[0]);
+         }
+         else
+         {
+            pb.skipToOneOf(Symbols::SEMI_COLON, ParseBuffer::Whitespace);
+            pb.data(value, anchor);
+         }
+
+         cookieList.push_back(Cookie(name, value));
+         DebugLog(<< "Cookie: " << Cookie(name, value));
+
+         if(!pb.eof() && *(pb.position()) == Symbols::SEMI_COLON[0])
+         {
+            pb.skipChar(Symbols::SEMI_COLON[0]);
+         }
+
+         pb.skipWhitespace();
+      }
+   }
+}
+
+/*
+ * Returns true if handshake complete, false if more bytes needed
+ * Sets dropConnection = true if an error occurs
+ */
+bool
+ConnectionBase::wsProcessHandshake(int bytesRead, bool &dropConnection)
+{
+   mConnState = WebSocket;
+   dropConnection = false;
+
+   if(mBufferPos + bytesRead > messageSizeMax)
+   {
+      WarningLog(<<"Too many bytes received during WS handshake, dropping connection.  Max message size = " << messageSizeMax);
+      dropConnection = true;
+      return false;
+   }
+
+   mMessage = new SipMessage(mWho.transport);
+   assert(mMessage);
+
+   mMessage->setSource(mWho);
+   mMessage->setTlsDomain(mWho.transport->tlsDomain());
+
+   if (!scanMsgHeader(bytesRead)) {
+      return false;
+   }
+
+   try
+   {
+      WsConnectionBase* wsConnectionBase = dynamic_cast<WsConnectionBase*>(this);
+      CookieList cookieList;
+      if(wsConnectionBase)
+      {
+         SharedPtr<WsCookieContext> wsCookieContext((WsCookieContext*)0);
+         if (mMessage->exists(h_Cookies))
+         {
+            WsBaseTransport* wst = dynamic_cast<WsBaseTransport*>(mTransport);
+            assert(wst);
+            try
+            {
+               wsParseCookies(cookieList, mMessage);
+               wsConnectionBase->setCookies(cookieList);
+               // Use of resip WsCookieContext capabilities is not mandatory,
+               // only try to use it if cookieContextFactory is available
+               if(wst->cookieContextFactory().get())
+               {
+                  wsCookieContext = wst->cookieContextFactory()->makeCookieContext(cookieList);
+                  wsConnectionBase->setWsCookieContext(wsCookieContext);
+               }
+            }
+            catch(ParseException& ex)
+            {
+               WarningLog(<<"Failed to parse cookies into WsCookieContext: " << ex);
+            }
+         }
+         SharedPtr<WsConnectionValidator> wsConnectionValidator = wsConnectionBase->connectionValidator();
+         if(wsConnectionValidator &&
+            (!wsCookieContext.get() || !wsConnectionValidator->validateConnection(*wsCookieContext)))
+         {
+            ErrLog(<<"WebSocket cookie validation failed, dropping connection");
+            // FIXME: should send back a HTTP error code:
+            //   400 if the cookie was not in the right syntax
+            //   403 if the cookie was well formed but rejected
+            //       due to expiry or a bad HMAC
+            delete mMessage;
+            mMessage = 0;
+            mBufferPos = 0;
+            dropConnection = true;
+            return false;
+         }
+      }
+
+      std::auto_ptr<Data> wsResponsePtr = makeWsHandshakeResponse();
+
+      if (wsResponsePtr.get())
+      {
+         DebugLog (<< "WebSocket upgrade accepted, cookie count = " << cookieList.size());
+
+         mOutstandingSends.push_back(new SendData(
+                  who(),
+                  *wsResponsePtr.get(),
+                  Data::Empty,
+                  Data::Empty,
+                  true));
+      }
+      else
+      {
+         ErrLog(<<"Failed to parse WebSocket initialization request");
+         delete mMessage;
+         mMessage = 0;
+         mBufferPos = 0;
+         dropConnection = true;
+         return false;
+      }
+   }
+   catch(resip::ParseException& e)
+   {
+      ErrLog(<<"Cannot auth request is missing " << e);
+      delete mMessage;
+      mMessage = 0;
+      mBufferPos = 0;
+      dropConnection = true;
+      return false;
+   }
+
+   delete mMessage;
+   mMessage=0;
+   mBufferPos = 0;
+
+   return true;
+}
+
+bool
+ConnectionBase::scanMsgHeader(int bytesRead)
+{
+   mMsgHeaderScanner.prepareForMessage(mMessage);
+   char *unprocessedCharPtr;
+   MsgHeaderScanner::ScanChunkResult scanResult = mMsgHeaderScanner.scanChunk(mBuffer, mBufferPos + bytesRead, &unprocessedCharPtr);
+   if (scanResult != MsgHeaderScanner::scrEnd)
+   {
+      if(scanResult != MsgHeaderScanner::scrNextChunk)
+      {
+         StackLog(<<"Failed to parse message, more bytes needed");
+         StackLog(<< Data(mBuffer, bytesRead));
+      }
+      delete mMessage;
+      mMessage=0;
+      mBufferPos += bytesRead;
+      return false;
+   }
+   return true;
+}
+
+std::auto_ptr<Data>
+ConnectionBase::makeWsHandshakeResponse()
+{
+   std::auto_ptr<Data> responsePtr(0);
+   if(isUsingSecWebSocketKey())
+   {
+      responsePtr.reset(new Data("HTTP/1.1 101 WebSocket Protocol Handshake\r\n"
+         "Upgrade: WebSocket\r\n"
+         "Connection: Upgrade\r\n"
+         "Sec-WebSocket-Protocol: sip\r\n"));
+
+      // Assuming that OpenSSL implementation of SHA1 is more effient than our internal one
+#ifdef USE_SSL
+      SHA1Stream wsSha1Stream;
+      wsSha1Stream << (mMessage->const_header(h_SecWebSocketKey).value() + Symbols::WebsocketMagicGUID);
+      Data wsAcceptKey = wsSha1Stream.getBin(160).base64encode();
+#else
+      SHA1 sha1;
+      sha1.update(mMessage->const_header(h_SecWebSocketKey).value().c_str());
+      sha1.update(Symbols::WebsocketMagicGUID);
+      Data wsAcceptKey = sha1.finalBin().base64encode();
+#endif
+      *responsePtr += "Sec-WebSocket-Accept: " + wsAcceptKey + "\r\n\r\n";
+   }
+   else if(isUsingDeprecatedSecWebSocketKeys())
+   {
+      ErrLog(<<"WS client wants to use depracated protocol version, unsupported");
+   }
+   else
+   {
+      ErrLog(<<"No SecWebSocketKey header");
+   }
+   return responsePtr;
+}
+
+bool ConnectionBase::isUsingDeprecatedSecWebSocketKeys()
+{
+   assert(mMessage);
+   return mMessage->exists(h_SecWebSocketKey1) &&
+      mMessage->exists(h_SecWebSocketKey2);
+}
+
+bool ConnectionBase::isUsingSecWebSocketKey()
+{
+   assert(mMessage);
+   return mMessage->exists(h_SecWebSocketKey);
+}
+
+bool
+ConnectionBase::wsProcessData(int bytesRead)
+{
+   bool dropConnection = false;
+   // Always consumes the whole buffer:
+   std::auto_ptr<Data> msg = mWsFrameExtractor.processBytes((UInt8*)mBuffer, bytesRead, dropConnection);
+
+   while(msg.get())
+   {
+      // mWsBuffer should now contain a discrete SIP message, let the
+      // stack go to work on it
+
+      if(msg->size() == 4 && memcmp(msg->data(), "\r\n\r\n", 4) == 0)
+      {
+         // sending a keep alive reply now
+         StackLog(<<"got a SIP ping embedded in WebSocket frame, replying");
+         onDoubleCRLF();
+         msg = mWsFrameExtractor.processBytes(0, 0, dropConnection);
+         continue;
+      }
+
+      mMessage = new SipMessage(mWho.transport);
+
+      mMessage->setSource(mWho);
+      mMessage->setTlsDomain(mWho.transport->tlsDomain());
+
+#ifdef USE_SSL
+      // Set TlsPeerName if message is from TlsConnection
+      TlsConnection *tlsConnection = dynamic_cast<TlsConnection *>(this);
+      if(tlsConnection)
+      {
+         std::list<Data> peerNameList;
+         tlsConnection->getPeerNames(peerNameList);
+         mMessage->setTlsPeerNames(peerNameList);
+      }
+#endif
+
+      WsConnectionBase *wsConnectionBase = dynamic_cast<WsConnectionBase *>(this);
+      if (wsConnectionBase)
+      {
+         mMessage->setWsCookies(wsConnectionBase->getCookies());
+         mMessage->setWsCookieContext(wsConnectionBase->getWsCookieContext());
+      }
+
+      Data::size_type msg_len = msg->size();
+      // cast permitted, as it is borrowed:
+      char *sipBuffer = (char *)msg->data();
+      mMessage->addBuffer(sipBuffer);
+      mMsgHeaderScanner.prepareForMessage(mMessage);
+      char *unprocessedCharPtr;
+      if (mMsgHeaderScanner.scanChunk(sipBuffer,
+                                 msg_len,
+                                 &unprocessedCharPtr) !=
+                    MsgHeaderScanner::scrEnd)
+      {
+         StackLog(<<"Scanner rejecting WebSocket SIP message as unparsable, length = " << msg_len);
+         StackLog(<< Data(sipBuffer, msg_len));
+         delete mMessage;
+         mMessage=0;
+      }
+
+      unsigned int used = unprocessedCharPtr - sipBuffer;
+      if (mMessage && (used < msg_len))
+      {
+         mMessage->setBody(sipBuffer+used, msg_len-used);
+      }
+
+      if (mMessage && !transport()->basicCheck(*mMessage))
+      {
+         delete mMessage;
+         mMessage = 0;
+      }
+
+      if (mMessage)
+      {
+         Transport::stampReceived(mMessage);
+         assert( mTransport );
+         mTransport->pushRxMsgUp(mMessage);
+         mMessage = 0;
+      }
+      else
+      {
+         // Something wrong...
+         ErrLog(<< "We don't have a valid SIP message, maybe drop the connection?");
+      }
+      msg = mWsFrameExtractor.processBytes(0, 0, dropConnection);
+   }
+
+   if(dropConnection)
+   {
+      return false;
+   }
+
+   return true;
+}
+
 #ifdef USE_SIGCOMP
 void
 ConnectionBase::decompressNewBytes(int bytesRead)
@@ -637,7 +976,7 @@ ConnectionBase::decompressNewBytes(int bytesRead)
       sc = 0;
     }
   }
-  delete uncompressed;
+  delete [] uncompressed;
 
   // If there was a decompression failure, let the other side know.
   osc::SigcompMessage *nack = mSigcompStack->getNack();
