@@ -436,22 +436,25 @@ DnsResult::lookupInternal(const Uri& uri)
                   mark=mInterface.getMarkManager().getMarkType(tuple);
                }
                
-               if(mark!=TupleMarkManager::OK && (mInterface.isSupported(TCP, V4) ||
-                                    mInterface.isSupported(TCP, V6)))
+               if (!mInterface.getUdpOnlyOnNumeric())
                {
-                  mTransport=TCP;
-                  mPort = getDefaultPort(mTransport,uri.port());
-                  tuple=Tuple(mTarget,mPort,mTransport,mTarget);
-                  mark=mInterface.getMarkManager().getMarkType(tuple);
-               }
-               
-               if(mark!=TupleMarkManager::OK && (mInterface.isSupported(TLS, V4) ||
-                                    mInterface.isSupported(TLS, V6)))
-               {
-                  mTransport=TLS;
-                  mPort = getDefaultPort(mTransport,uri.port());
-                  tuple=Tuple(mTarget,mPort,mTransport,mTarget);
-                  mark=mInterface.getMarkManager().getMarkType(tuple);
+                  if(mark!=TupleMarkManager::OK && (mInterface.isSupported(TCP, V4) ||
+                           mInterface.isSupported(TCP, V6)))
+                  {
+                     mTransport=TCP;
+                     mPort = getDefaultPort(mTransport,uri.port());
+                     tuple=Tuple(mTarget,mPort,mTransport,mTarget);
+                     mark=mInterface.getMarkManager().getMarkType(tuple);
+                  }
+
+                  if(mark!=TupleMarkManager::OK && (mInterface.isSupported(TLS, V4) ||
+                           mInterface.isSupported(TLS, V6)))
+                  {
+                     mTransport=TLS;
+                     mPort = getDefaultPort(mTransport,uri.port());
+                     tuple=Tuple(mTarget,mPort,mTransport,mTarget);
+                     mark=mInterface.getMarkManager().getMarkType(tuple);
+                  }
                }
             }
             
@@ -556,6 +559,10 @@ DnsResult::getDefaultPort(TransportType transport, int port)
          case TLS:
          case DTLS:
             return Symbols::DefaultSipsPort;
+         case WS:
+            return Symbols::SipWsPort;
+         case WSS:
+            return Symbols::SipWssPort;
          default:
             ErrLog( << "Should not get this - unknown transport" );
             return Symbols::DefaultSipPort; // !cj! todo - remove 
@@ -1377,7 +1384,6 @@ DnsResult::onDnsResult(const DNSResult<DnsNaptrRecord>& result)
    }
 
    onNaptrResult(result);
-  
 }
 
 void DnsResult::onDnsResult(const DNSResult<DnsCnameRecord>& result)
@@ -1475,3 +1481,5 @@ resip::operator<<(EncodeStream& strm, const resip::DnsResult::SRV& srv)
  * <http://www.vovida.org/>.
  *
  */
+
+// vim: softtabstop=3:shiftwidth=3:expandtab
