@@ -30,6 +30,7 @@
 #include "rutil/AsyncProcessHandler.hxx"
 #include "resip/stack/TcpTransport.hxx"
 #include "resip/stack/UdpTransport.hxx"
+#include "resip/stack/WsTransport.hxx"
 #include "resip/stack/TransactionUser.hxx"
 #include "resip/stack/TransactionUserMessage.hxx"
 #include "resip/stack/TransactionControllerThread.hxx"
@@ -40,6 +41,7 @@
 #include "resip/stack/ssl/Security.hxx"
 #include "resip/stack/ssl/DtlsTransport.hxx"
 #include "resip/stack/ssl/TlsTransport.hxx"
+#include "resip/stack/ssl/WssTransport.hxx"
 #endif
 
 #if defined(WIN32) && !defined(__GNUC__)
@@ -299,8 +301,11 @@ SipStack::addTransport( TransportType protocol,
                         const Data& privateKeyPassPhrase,
                         SecurityTypes::SSLType sslType,
                         unsigned transportFlags,
+                        const Data& certificateFilename, const Data& privateKeyFilename,
                         SecurityTypes::TlsClientVerificationMode cvm,
-                        bool useEmailAsSIP)
+                        bool useEmailAsSIP,
+                        SharedPtr<WsConnectionValidator> wsConnectionValidator,
+                        SharedPtr<WsCookieContextFactory> wsCookieContextFactory)
 {
    assert(!mShuttingDown);
 
@@ -354,7 +359,8 @@ SipStack::addTransport( TransportType protocol,
                                          *mCompression,
                                          transportFlags,
                                          cvm,
-                                         useEmailAsSIP);
+                                         useEmailAsSIP,
+                                         certificateFilename, privateKeyFilename);
 #else
             CritLog (<< "TLS not supported in this stack. You don't have openssl");
             assert(0);
@@ -369,10 +375,46 @@ SipStack::addTransport( TransportType protocol,
                                           *mSecurity,
                                           sipDomainname,
                                           mSocketFunc,
-                                          *mCompression);
+                                          *mCompression,
+                                          certificateFilename, privateKeyFilename);
 #else
             CritLog (<< "DTLS not supported in this stack.");
             assert(0);
+#endif
+            break;
+
+         case WS:
+            transport = new WsTransport(stateMacFifo, 
+                  port,
+                  version,
+                  ipInterface,
+                  mSocketFunc,
+                  *mCompression,
+                  transportFlags,
+                  wsConnectionValidator,
+                  wsCookieContextFactory);
+            break;
+
+         case WSS:
+#if defined( USE_SSL )
+            transport = new WssTransport(stateMacFifo,
+                  port,
+                  version,
+                  ipInterface,
+                  *mSecurity,
+                  sipDomainname,
+                  sslType,
+                  mSocketFunc,
+                  *mCompression,
+                  transportFlags,
+                  cvm,
+                  useEmailAsSIP,
+                  wsConnectionValidator,
+                  wsCookieContextFactory,
+                  certificateFilename, privateKeyFilename);
+#else
+            CritLog (<< "WSS not supported in this stack. You don't have openssl");
+            assert(0);
 #endif
             break;
          default:
@@ -438,12 +480,10 @@ SipStack::addAlias(const Data& domain, int port)
    assert(!mShuttingDown);
    mDomains.insert(domain + ":" + Data(portToUse));
 
-
    if(mUri.host().empty())
    {
       mUri.host()=*mDomains.begin();
    }
-
 }
 
 Data
