@@ -26,6 +26,7 @@
 #include "resip/stack/TcpBaseTransport.hxx"
 #include "resip/stack/TcpTransport.hxx"
 #include "resip/stack/UdpTransport.hxx"
+#include "resip/stack/WsTransport.hxx"
 #include "resip/stack/Uri.hxx"
 
 #include "rutil/DataStream.hxx"
@@ -46,6 +47,7 @@
 #include "resip/stack/ssl/DtlsTransport.hxx"
 #include "resip/stack/ssl/Security.hxx"
 #include "resip/stack/ssl/TlsTransport.hxx"
+#include "resip/stack/ssl/WssTransport.hxx"
 #endif
 
 #ifdef WIN32
@@ -204,6 +206,16 @@ TransportSelector::addTransportInternal(std::auto_ptr<Transport> autoTransport)
       assert(dynamic_cast<DtlsTransport*>(transport));
    }
 #endif
+#endif
+   else if(transport->transport()==WS)
+   {
+      assert(dynamic_cast<WsTransport*>(transport));
+   }
+#ifdef USE_SSL
+   else if(transport->transport()==WSS)
+   {
+      assert(dynamic_cast<WssTransport*>(transport));
+   }
 #endif
    else
    {
@@ -218,6 +230,7 @@ TransportSelector::addTransportInternal(std::auto_ptr<Transport> autoTransport)
    {
       case UDP:
       case TCP:
+      case WS:
       {
          assert(mExactTransports.find(tuple) == mExactTransports.end() &&
                 mAnyInterfaceTransports.find(tuple) == mAnyInterfaceTransports.end());
@@ -243,6 +256,7 @@ TransportSelector::addTransportInternal(std::auto_ptr<Transport> autoTransport)
       break;
       case TLS:
       case DTLS:
+      case WSS:
       {
          TlsTransportKey key(transport->tlsDomain(),transport->transport(),transport->ipVersion());
          mTlsTransports[key]=transport;
@@ -466,6 +480,7 @@ TransportSelector::dnsResolve(DnsResult* result,
    }
 }
 
+// FIXME: maybe this should move to rutil/TransportType?
 bool isDgramTransport (TransportType type)
 {
    static const bool unknown_transport = false;
@@ -479,6 +494,8 @@ bool isDgramTransport (TransportType type)
 
       case TCP:
       case TLS:
+      case WS:
+      case WSS:
          return   false;
 
       default:
@@ -620,9 +637,9 @@ TransportSelector::determineSourceInterface(SipMessage* msg, const Tuple& target
          GenericIPAddress addr = WinCompat::determineSourceInterface(target.toGenericIPAddress());
          source.setSockaddr(addr);
       }
-      catch (WinCompat::Exception&)
+      catch (WinCompat::Exception& ex)
       {
-         ErrLog (<< "Can't find source interface to use");
+         ErrLog (<< "Can't find source interface to use: " << ex);
          throw Transport::Exception("Can't find source interface", __FILE__, __LINE__);
       }
 #else
@@ -755,7 +772,7 @@ TransportSelector::determineSourceInterface(SipMessage* msg, const Tuple& target
 
 // !jf! there may be an extra copy of a tuple here. can probably get rid of it
 // but there are some const issues.
-bool
+TransportSelector::TransmitState
 TransportSelector::transmit(SipMessage* msg, Tuple& target, SendData* sendData)
 {
    assert(msg);
@@ -1049,7 +1066,6 @@ TransportSelector::transmit(SipMessage* msg, Tuple& target, SendData* sendData)
                      contact.uri().remove(p_addTransport);
                   }
                }
-
             }
          }
 
@@ -1167,13 +1183,13 @@ TransportSelector::transmit(SipMessage* msg, Tuple& target, SendData* sendData)
          }
 
          transport->send(send);
-         return true;
+         return Sent;
       }
       else
       {
          InfoLog (<< "tid=" << msg->getTransactionId() << " failed to find a transport to " << target);
          mStateMacFifo.add(new TransportFailure(msg->getTransactionId(), transportFailureReason));
-         return false;
+         return Unsent;
       }
 
    }
@@ -1181,7 +1197,7 @@ TransportSelector::transmit(SipMessage* msg, Tuple& target, SendData* sendData)
    {
       InfoLog (<< "tid=" << msg->getTransactionId() << " no route to target: " << target);
       mStateMacFifo.add(new TransportFailure(msg->getTransactionId(), TransportFailure::NoRoute));
-      return false;
+      return Unsent;
    }
 }
 
