@@ -1357,6 +1357,8 @@ defineParam(extension, "ext", DataParameter, "RFC 3966"); // Token is used when
 defineParam(sigcompId, "sigcomp-id", QuotedDataParameter, "RFC 5049");
 defineParam(rinstance, "rinstance", DataParameter, "proprietary (resip)");
 defineParam(addTransport, "addTransport", ExistsParameter, "RESIP INTERNAL");
+defineParam(wsSrcIp, "ws-src-ip", DataParameter, "RESIP INTERNAL (WebSocket)");
+defineParam(wsSrcPort, "ws-src-port", UInt32Parameter, "RESIP INTERNAL (WebSocket)");
 
 #undef defineParam
 
