@@ -80,13 +80,18 @@ DtlsTransport::DtlsTransport(Fifo<TransactionMessage>& fifo,
                              Security& security,
                              const Data& sipDomain,
                              AfterSocketCreationFuncPtr socketFunc,
-                             Compression& compression)
- : UdpTransport( fifo, portNum, version,
-   StunDisabled, interfaceObj, socketFunc, compression ),
+                             Compression& compression,
+                             const Data& certificateFilename, const Data& privateKeyFilename)
+ : UdpTransport( fifo, portNum, version, StunDisabled, interfaceObj, socketFunc, compression ),
    mTimer( mHandshakePending ),
    mSecurity( &security ),
    mDomain(sipDomain)
 {
+   // Note on AfterSocketCreateFuncPtr:  because this class uses UdpTransport the bind operation 
+   //   is called in the UdpTransport constructor and the transport type passed to AfterSocketCreationFuncPtr
+   //   will end up being UDP and not DTLS.  TODO - this should be fixed.  Creating a UdpBaseTransport is one
+   //   solution that would align with the TCP flavour of transports.
+   
    setTlsDomain(sipDomain);
    InfoLog ( << "Creating DTLS transport host=" << interfaceObj
              << " port=" << mTuple.getPort()
@@ -94,10 +99,10 @@ DtlsTransport::DtlsTransport(Fifo<TransactionMessage>& fifo,
 
    mTxFifo.setDescription("DtlsTransport::mTxFifo");
 
-   mTuple.setType( transport() );
+   mTuple.setType( DTLS );
 
-   mClientCtx = mSecurity->createDomainCtx(DTLSv1_client_method(), Data::Empty) ;
-   mServerCtx = mSecurity->createDomainCtx(DTLSv1_server_method(), sipDomain) ;
+   mClientCtx = mSecurity->createDomainCtx(DTLSv1_client_method(), Data::Empty, certificateFilename, privateKeyFilename) ;
+   mServerCtx = mSecurity->createDomainCtx(DTLSv1_server_method(), sipDomain, certificateFilename, privateKeyFilename) ;
    assert( mClientCtx ) ;
    assert( mServerCtx ) ;
 
@@ -170,14 +175,15 @@ DtlsTransport::_read( FdSet& fdset )
    if (len == 0 || len == SOCKET_ERROR)
    {
       delete [] buffer ;
-      buffer = 0 ;
+      delete [] pt;
       return ;
    }
 
    if ( len + 1 >= UdpTransport::MaxBufferSize )
    {
       InfoLog (<<"Datagram exceeded max length "<<UdpTransport::MaxBufferSize ) ;
-      delete [] buffer ; buffer = 0 ;
+      delete [] buffer ; 
+      delete [] pt;
       return ;
    }
 
@@ -282,10 +288,14 @@ DtlsTransport::_read( FdSet& fdset )
    }
 
    if ( len <= 0 )
+   {
        return ;
+   }
 
    if ( SSL_in_init( ssl ) )
+   {
       mTimer.add( ssl, DtlsReceiveTimeout ) ;
+   }
 
 #ifdef USE_SIGCOMP
    osc::StateChanges *sc = 0;
