@@ -112,7 +112,10 @@ verifyCallback(int iInCode, X509_STORE_CTX *pInStore)
 
    snprintf(cBuf2, 500, ", depth=%d %s\n", iDepth, cBuf1);
    if(!iInCode)
-      ErrLog(<< "Error when verifying server's chain of certificates: " << X509_verify_cert_error_string(pInStore->error) << cBuf2 );
+   {
+      ErrLog(<< "Error when verifying peer's chain of certificates: " << X509_verify_cert_error_string(pInStore->error) << cBuf2 );
+      DebugLog(<<"additional validation checks may have failed but only one is ever logged - please check peer certificate carefully");
+   }
  
    return iInCode;
 }
@@ -142,9 +145,8 @@ Security::Security(const Data& directory, const CipherList& cipherSuite) :
    BaseSecurity(cipherSuite), 
    mPath(directory)
 {
-   // since the preloader won't work otherwise and VERY difficult to figure
-   // out. 
-   if ( !mPath.postfix(Symbols::SLASH))
+   // since the preloader won't work otherwise and VERY difficult to figure out.
+   if (!mPath.empty() && !mPath.postfix(Symbols::SLASH))
    {
       mPath += Symbols::SLASH;
    }
@@ -262,7 +264,7 @@ Security::preload()
 }
 
 SSL_CTX* 
-Security::createDomainCtx(const SSL_METHOD* method, const Data& domain)
+Security::createDomainCtx(const SSL_METHOD* method, const Data& domain, const Data& certificateFilename, const Data& privateKeyFilename)
 {
 #if (OPENSSL_VERSION_NUMBER >= 0x1000000fL )
    SSL_CTX* ctx = SSL_CTX_new(method);
@@ -285,14 +287,14 @@ Security::createDomainCtx(const SSL_METHOD* method, const Data& domain)
    // Load domain cert chain and private key
    if(!domain.empty())
    {
-      Data certFilename(mPath + pemTypePrefixes(DomainCert) + domain + PEM);
+      Data certFilename(certificateFilename.empty() ? mPath + pemTypePrefixes(DomainCert) + domain + PEM : certificateFilename);
       if(SSL_CTX_use_certificate_chain_file(ctx, certFilename.c_str()) != 1)
       {
          ErrLog (<< "Error reading domain chain file " << certFilename);
          SSL_CTX_free(ctx);
          throw BaseSecurity::Exception("Failed opening PEM chain file", __FILE__,__LINE__);
       }
-      Data keyFilename(mPath + pemTypePrefixes(DomainPrivateKey) + domain + PEM);
+      Data keyFilename(privateKeyFilename.empty() ? mPath + pemTypePrefixes(DomainPrivateKey) + domain + PEM : privateKeyFilename);
       if(SSL_CTX_use_PrivateKey_file(ctx, keyFilename.c_str(), SSL_FILETYPE_PEM) != 1)
       {
          ErrLog (<< "Error reading domain private key file " << keyFilename);
@@ -1115,14 +1117,14 @@ BaseSecurity::addRootCertPEM(const Data& x509PEMEncodedRootCerts)
 void
 BaseSecurity::addDomainCertPEM(const Data& domainName, const Data& certPEM)
 {
-   addCertPEM(DomainCert, domainName, certPEM, true);
+   addCertPEM(DomainCert, domainName, certPEM, false);
 }
 
 
 void
 BaseSecurity::addDomainCertDER(const Data& domainName, const Data& certDER)
 {
-   addCertDER(DomainCert, domainName, certDER, true);
+   addCertDER(DomainCert, domainName, certDER, false);
 }
 
 
@@ -1150,7 +1152,7 @@ BaseSecurity::getDomainCertDER(const Data& domainName) const
 void
 BaseSecurity::addDomainPrivateKeyPEM(const Data& domainName, const Data& privateKeyPEM)
 {
-   addPrivateKeyPEM(DomainPrivateKey, domainName, privateKeyPEM, true);
+   addPrivateKeyPEM(DomainPrivateKey, domainName, privateKeyPEM, false);
 }
 
 
@@ -1178,14 +1180,14 @@ BaseSecurity::getDomainPrivateKeyPEM(const Data& domainName) const
 void
 BaseSecurity::addUserCertPEM(const Data& aor, const Data& certPEM)
 {
-   addCertPEM(UserCert, aor, certPEM, true);
+   addCertPEM(UserCert, aor, certPEM, false);
 }
 
 
 void
 BaseSecurity::addUserCertDER(const Data& aor, const Data& certDER)
 {
-   addCertDER(UserCert, aor, certDER, true);
+   addCertDER(UserCert, aor, certDER, false);
 }
 
 
@@ -1273,14 +1275,14 @@ BaseSecurity::getUserPassPhrase(const Data& aor) const
 void
 BaseSecurity::addUserPrivateKeyPEM(const Data& aor, const Data& cert)
 {
-   addPrivateKeyPEM(UserPrivateKey, aor, cert, true);
+   addPrivateKeyPEM(UserPrivateKey, aor, cert, false);
 }
 
 
 void
 BaseSecurity::addUserPrivateKeyDER(const Data& aor, const Data& cert)
 {
-   addPrivateKeyDER(UserPrivateKey, aor, cert, true);
+   addPrivateKeyDER(UserPrivateKey, aor, cert, false);
 }
 
 
