@@ -25,6 +25,34 @@ using namespace resip;
 
 #define RESIPROCATE_SUBSYSTEM Subsystem::TRANSPORT
 
+inline bool handleOpenSSLErrorQueue(int ret, unsigned long err, const char* op)
+{
+   bool hadReason = false;
+   while (true)
+   {
+      const char* file;
+      int line;
+
+      unsigned long code = ERR_get_error_line(&file,&line);
+      if ( code == 0 )
+      {
+         break;
+      }
+
+      char buf[256];
+      ERR_error_string_n(code,buf,sizeof(buf));
+      ErrLog( << buf  );
+      DebugLog( << "Error code = " << code << " file=" << file << " line=" << line );
+      hadReason = true;
+   }
+   ErrLog( << "Got TLS " << op << " error=" << err << " ret=" << ret  );
+   if(!hadReason)
+   {
+      WarningLog(<<"no reason found with ERR_get_error_line");
+   }
+   return hadReason;
+}
+
 TlsConnection::TlsConnection( Transport* transport, const Tuple& tuple, 
                               Socket fd, Security* security, 
                               bool server, Data domain,  SecurityTypes::SSLType sslType ,
@@ -109,7 +137,27 @@ TlsConnection::TlsConnection( Transport* transport, const Tuple& tuple,
 TlsConnection::~TlsConnection()
 {
 #if defined(USE_SSL)
-   SSL_shutdown(mSsl);
+   ERR_clear_error();
+   int ret = SSL_shutdown(mSsl);
+   if(ret < 0)
+   {
+      int err = SSL_get_error(mSsl, ret);
+      switch (err)
+      {
+         case SSL_ERROR_WANT_READ:
+         case SSL_ERROR_WANT_WRITE:
+         case SSL_ERROR_NONE:
+            {
+               // WANT_READ or WANT_WRITE can arise for bi-directional shutdown on
+               // non-blocking sockets, safe to ignore
+               StackLog( << "Got TLS shutdown error condition of " << err  );
+            }
+            break;
+         default:
+            ErrLog(<<"Unexpected error in SSL_shutdown");
+            handleOpenSSLErrorQueue(ret, err, "SSL_shutdown");
+      }
+   }
    SSL_free(mSsl);
 #endif // USE_SSL   
 }
@@ -267,29 +315,7 @@ TlsConnection::checkState()
                DebugLog(<<"unrecognised/unhandled SSL_get_error result: " << err);
             }
             ErrLog( << "TLS handshake failed ");
-            bool hadReason = false;
-            while (true)
-            {
-               const char* file;
-               int line;
-
-               unsigned long code = ERR_get_error_line(&file,&line);
-               if ( code == 0 )
-               {
-                  break;
-               }
-
-               char buf[256];
-               ERR_error_string_n(code,buf,sizeof(buf));
-               ErrLog( << buf  );
-               ErrLog( << "Error code = "
-                        << code << " file=" << file << " line=" << line );
-               hadReason = true;
-            }
-            if(!hadReason)
-            {
-               WarningLog(<<"no reason found with ERR_get_error_line");
-            }
+            handleOpenSSLErrorQueue(ok, err, "SSL_do_handshake");
             mBio = NULL;
             mTlsState = Broken;
             return mTlsState;
@@ -372,29 +398,37 @@ TlsConnection::read(char* buf, int count )
    int bytesRead = SSL_read(mSsl,buf,count);
    StackLog(<< "SSL_read returned " << bytesRead << " bytes [" << Data(Data::Borrow, buf, (bytesRead > 0)?(bytesRead):(0)) << "]");
 
-   int bytesPending = SSL_pending(mSsl);
-
-   if ((bytesRead > 0) && (bytesPending > 0))
+   if (bytesRead > 0)
    {
-      char* buffer = getWriteBufferForExtraBytes(bytesPending);
-      if (buffer)
+      int bytesPending = SSL_pending(mSsl);
+      if (bytesPending > 0)
       {
-         StackLog(<< "reading remaining buffered bytes");
-         bytesPending = SSL_read(mSsl, buffer, bytesPending);
-         StackLog(<< "SSL_read returned  " << bytesPending << " bytes [" << Data(Data::Borrow, buffer, (bytesPending > 0)?(bytesPending):(0)) << "]");
-         
-         if (bytesPending > 0)
+         char* buffer = getWriteBufferForExtraBytes(bytesPending);
+         if (buffer)
          {
-            bytesRead += bytesPending;
+            StackLog(<< "reading remaining buffered bytes");
+            bytesPending = SSL_read(mSsl, buffer, bytesPending);
+            StackLog(<< "SSL_read returned  " << bytesPending << " bytes [" << Data(Data::Borrow, buffer, (bytesPending > 0)?(bytesPending):(0)) << "]");
+            
+            if (bytesPending > 0)
+            {
+               bytesRead += bytesPending;
+            }
+            else
+            {
+               bytesRead = bytesPending;
+            }
          }
          else
          {
-            bytesRead = bytesPending;
+            assert(0);
          }
       }
-      else
+      else if (bytesPending < 0)
       {
-         assert(0);
+         int err = SSL_get_error(mSsl, bytesPending);
+         handleOpenSSLErrorQueue(bytesPending, err, "SSL_pending");
+         return -1;
       }
    }
 
@@ -413,9 +447,11 @@ TlsConnection::read(char* buf, int count )
          break;
          default:
          {
-            char buf[256];
-            ERR_error_string_n(err,buf,sizeof(buf));
-            ErrLog( << "Got TLS read ret=" << bytesRead << " error=" << err  << " " << buf << (err==5?" - intermediate certificates may be missing from local PEM file" : "") );
+            handleOpenSSLErrorQueue(bytesRead, err, "SSL_read");
+            if(err == 5)
+            {
+               WarningLog(<<"err=5 sometimes indicates that intermediate certificates may be missing from local PEM file");
+            }
             return -1;
          }
          break;
@@ -498,23 +534,7 @@ TlsConnection::write( const char* buf, int count )
          break;
          default:
          {
-            while (true)
-            {
-               const char* file;
-               int line;
-               
-               unsigned long code = ERR_get_error_line(&file,&line);
-               if ( code == 0 )
-               {
-                  break;
-               }
-               
-               char buf[256];
-               ERR_error_string_n(code,buf,sizeof(buf));
-               ErrLog( << buf  );
-               DebugLog( << "Error code = " << code << " file=" << file << " line=" << line );
-            }
-            ErrLog( << "Got TLS write error=" << err << " ret=" << ret  );
+            handleOpenSSLErrorQueue(ret, err, "SSL_write");
             return -1;
          }
          break;
@@ -563,6 +583,13 @@ TlsConnection::isGood() // has data that can be read
    }
 
    int mode = SSL_get_shutdown(mSsl);
+   if ( mode < 0 )
+   {
+      int err = SSL_get_error(mSsl, mode);
+      handleOpenSSLErrorQueue(mode, err, "SSL_get_shutdown");
+      return false;
+   }
+
    if ( mode != 0 ) 
    {
       return false;
