@@ -56,6 +56,27 @@ const bool DataHelper::isCharHex[256] =
 //must be lowercase for MD5
 static const char hexmap[] = "0123456789abcdef";
 
+static const char inversehexmap[] = {
+    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,     // 0 - 
+    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+    -1, -1, -1, -1, -1, -1, -1, -1,             //   - 47
+
+          0, 1, 2, 3, 4, 5, 6, 7, 8, 9,         // 48 ...
+
+                                     -1, -1,    // 58 -
+    -1, -1, -1, -1, -1,                         //  -64
+
+          10, 11, 12, 13, 14, 15,               // 65 ...
+
+        -1, -1, -1, -1, -1, -1, -1, -1, -1,     // 71 ...
+    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
+    -1, -1, -1, -1, -1, -1, -1,                 //   - 96
+
+          10, 11, 12, 13, 14, 15                // 97 ...
+};
+
 int 
 hexpair2int(char high, char low)
 {
@@ -290,6 +311,15 @@ Data::Data(ShareEnum se, const char* buffer, size_type length)
    assert(buffer);
 }
 
+Data::Data(ShareEnum se, const char* buffer, size_type length, size_type capacity)
+   : mBuf(const_cast<char*>(buffer)),
+     mSize(length),
+     mCapacity(capacity),
+     mShareEnum(se)
+{
+   assert(buffer);
+}
+
 Data::Data(ShareEnum se, const char* buffer)
    : mBuf(const_cast<char*>(buffer)),
      mSize(strlen(buffer)),
@@ -336,17 +366,22 @@ Data::Data(Data &&data)
 #endif
 
 // -2147483646
-static const int IntMaxSize = 12;
+static const int Int32MaxSize = 11;
 
-Data::Data(int val)
-   : mBuf(IntMaxSize > LocalAlloc 
-          ? new char[IntMaxSize + 1]
+static const int MaxLongSize = (sizeof(unsigned long)/sizeof(int))*Int32MaxSize;
+
+// 18446744073709551615
+static const int UInt64MaxSize = 20;
+
+Data::Data(Int32 val)
+   : mBuf(Int32MaxSize > LocalAlloc 
+          ? new char[Int32MaxSize + 1]
           : mPreBuffer),
      mSize(0),
-     mCapacity(IntMaxSize > LocalAlloc
-               ? IntMaxSize
+     mCapacity(Int32MaxSize > LocalAlloc
+               ? Int32MaxSize
                : LocalAlloc),
-     mShareEnum(IntMaxSize > LocalAlloc ? Take : Borrow)
+     mShareEnum(Int32MaxSize > LocalAlloc ? Take : Borrow)
 {
    if (val == 0)
    {
@@ -358,7 +393,7 @@ Data::Data(int val)
 
    bool neg = false;
    
-   int value = val;
+   Int32 value = val;
    if (value < 0)
    {
       value = -value;
@@ -366,7 +401,7 @@ Data::Data(int val)
    }
 
    int c = 0;
-   int v = value;
+   Int32 v = value;
    while (v /= 10)
    {
       ++c;
@@ -393,47 +428,8 @@ Data::Data(int val)
    }
 }
 
-static const int MaxLongSize = (sizeof(unsigned long)/sizeof(int))*IntMaxSize;
-Data::Data(unsigned long value)
-   : mBuf(MaxLongSize > LocalAlloc 
-          ? new char[MaxLongSize + 1]
-          : mPreBuffer),
-     mSize(0),
-     mCapacity(MaxLongSize > LocalAlloc
-               ? MaxLongSize
-               : LocalAlloc),
-     mShareEnum(MaxLongSize > LocalAlloc ? Take : Borrow)
-{
-   if (value == 0)
-   {
-      mBuf[0] = '0';
-      mBuf[1] = 0;
-      mSize = 1;
-      return;
-   }
-
-   int c = 0;
-   unsigned long v = value;
-   while (v /= 10)
-   {
-      ++c;
-   }
-
-   mSize = c+1;
-   mBuf[c+1] = 0;
-   
-   v = value;
-   while (v)
-   {
-      unsigned int digit = v%10;
-      unsigned char d = (char)digit;
-      mBuf[c--] = '0' + d;
-      v /= 10;
-   }
-}
-
 #ifndef RESIP_FIXED_POINT
-static const int DoubleMaxSize = MaxLongSize + Data::MaxDigitPrecision;
+static const int DoubleMaxSize = UInt64MaxSize + Data::MaxDigitPrecision;
 Data::Data(double value, 
            Data::DoubleDigitPrecision precision)
    : mBuf(DoubleMaxSize + precision > LocalAlloc 
@@ -456,7 +452,7 @@ Data::Data(double value,
       v = -v;
    }
 
-   Data m((unsigned long)v);
+   Data m((UInt64)v);
 
    // remainder
    v = v - floor(v);
@@ -521,15 +517,15 @@ Data::Data(double value,
 }
 #endif
 
-Data::Data(unsigned int value)
-   : mBuf(IntMaxSize > LocalAlloc 
-          ? new char[IntMaxSize + 1]
+Data::Data(UInt32 value)
+   : mBuf(Int32MaxSize > LocalAlloc 
+          ? new char[Int32MaxSize + 1]
           : mPreBuffer),
      mSize(0),
-     mCapacity(IntMaxSize > LocalAlloc
-               ? IntMaxSize
+     mCapacity(Int32MaxSize > LocalAlloc
+               ? Int32MaxSize
                : LocalAlloc),
-     mShareEnum(IntMaxSize > LocalAlloc ? Take : Borrow)
+     mShareEnum(Int32MaxSize > LocalAlloc ? Take : Borrow)
 {
    if (value == 0)
    {
@@ -540,7 +536,7 @@ Data::Data(unsigned int value)
    }
 
    int c = 0;
-   unsigned long v = value;
+   UInt32 v = value;
    while (v /= 10)
    {
       ++c;
@@ -559,9 +555,6 @@ Data::Data(unsigned int value)
    }
 }
 
-// 18446744073709551615
-static const int UInt64MaxSize = 20;
-
 Data::Data(UInt64 value)
    : mBuf(UInt64MaxSize > LocalAlloc 
           ? new char[UInt64MaxSize + 1]
@@ -726,7 +719,7 @@ bool
 resip::operator==(const Data& lhs, const char* rhs)
 {
    assert(rhs); // .dlb. not consistent with constructor
-   if (memcmp(lhs.mBuf, rhs, lhs.mSize) != 0)
+   if (strncmp(lhs.mBuf, rhs, lhs.mSize) != 0)
    {
       return false;
    }
@@ -1514,6 +1507,42 @@ Data::hex() const
    return ret;
 }
 
+Data
+Data::fromHex() const
+{
+   bool oddSize = mSize & 1;
+   size_type resultSize = (mSize + (mSize & 1)) / 2;
+   Data ret(resultSize, Data::Preallocate);
+
+   const unsigned char* p = (unsigned char*)mBuf;
+   char* r = ret.mBuf;
+   size_type i = 0;
+   if(oddSize)
+   {
+      if(!isHex(*p))
+      {
+         throw DataException("Encountered non-hex digit",
+                                    __FILE__,__LINE__);
+      }
+      *r++ = inversehexmap[*p++];
+      i++;
+   }
+   for ( ; i < mSize; i+=2)
+   {
+      const unsigned char high = *p++;
+      const unsigned char low = *p++;
+      if(!isHex(high) || !isHex(low))
+      {
+         throw DataException("Encountered non-hex digit",
+                                    __FILE__,__LINE__);
+      }
+
+      *r++ = (inversehexmap[high] << 4) + inversehexmap[low];
+   }
+   ret.mSize = resultSize;
+   return ret;
+}
+
 Data&
 Data::lowercase()
 {
