@@ -300,7 +300,7 @@ Log::toLevel(const Data& l)
    int i=0;
    while (strlen(mDescriptions[i]))
    {
-      if (strcmp(pri.c_str(), mDescriptions[i]) == 0)
+      if (isEqualNoCase(pri, Data(mDescriptions[i])))
       {
          return Level(i-1);
       }
@@ -368,12 +368,12 @@ Log::tags(Log::Level level,
 #else // #if defined( WIN32 ) || defined( __APPLE__ )
    if(resip::Log::getLoggerData().type() == Syslog)
    {
-      strm << mDescriptions[level+1] << Log::delim
-           << timestamp(ts) << Log::delim
+      strm // << mDescriptions[level+1] << Log::delim
+   //        << timestamp(ts) << Log::delim
    //        << mHostname << Log::delim
-           << mAppName << Log::delim
+   //        << mAppName << Log::delim
            << subsystem << Log::delim
-           << mPid << Log::delim
+   //        << mPid << Log::delim
            << pthread_self() << Log::delim
            << pfile << ":" << line;
    }
